import React from 'react'
import { Icon } from '@pubsweet/ui'
import { th, darken } from '@pubsweet/ui-toolkit'
import styled, { withTheme } from 'styled-components'
import { states } from 'config'
import MetaSec from './MetaSec'

const Checklist = styled.div`
  display: flex;
  flex-direction: row;
  align-items: flex-start;
  border-right: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
  @media screen and (max-width: 970px) {
    flex-wrap: wrap;
  }
`
const Col = styled.div`
  flex: ${props => props.flex};
  display: flex;
  flex-direction: column;
  border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
  border-right: 0;

  & > div {
    flex: 1 1 calc(${th('gridUnit')} * 6.25);
    text-align: center;
    padding: ${th('gridUnit')};
    &:first-child {
      flex: 1;
      font-weight: 600;
      background-color: ${darken('colorBackgroundHue', 7)};
      border-bottom: ${th('borderWidth')} ${th('borderStyle')}
        ${th('colorBorder')};
    }
  }

  @media screen and (max-width: 970px) {
    div {
      font-size: ${th('fontSizeBaseSmall')};
      white-space: nowrap;
    }
  }
`

const StatusIcon = withTheme(({ theme }) => (
  <Icon color={theme.colorSuccess} size={3}>
    check
  </Icon>
))

const Status = ({ current, mark }) => {
  const curr = states.indexOf(current)
  const compare = states.indexOf(mark)
  return (
    <div>
      {curr === compare && '...'}
      {curr > compare && <StatusIcon />}
    </div>
  )
}

const CheckCitation = ({ manuscript }) => {
  const { meta } = manuscript
  const { articleIds, volume, location, publicationDates, issue } = meta
  const hasPmid = articleIds && articleIds.some(aid => aid.pubIdType === 'pmid')
  const hasPprid =
    articleIds && articleIds.some(aid => aid.pubIdType === 'pprid')
  const hasCitation =
    volume &&
    publicationDates &&
    publicationDates.length > 0 &&
    location &&
    (location.fpage || location.elocationId)
  return (
    <div>
      {hasPmid || hasPprid || hasCitation ? (
        <React.Fragment>
          {(hasPmid && (
            <React.Fragment>
              {volume || issue ? <StatusIcon /> : '...'}
            </React.Fragment>
          )) ||
            (hasPprid && (
              <React.Fragment>
                {publicationDates && publicationDates.length > 0 ? (
                  <StatusIcon />
                ) : (
                  '...'
                )}
              </React.Fragment>
            )) ||
            (hasCitation && <StatusIcon />)}
        </React.Fragment>
      ) : (
        '...'
      )}
    </div>
  )
}

const QuickView = ({ manuscript }) => {
  const preprint =
    manuscript.organization &&
    manuscript.organization.name === 'Europe PMC Preprints'
  const indexOnly =
    manuscript.ebiState && manuscript.ebiState.includes('index only')
  return (
    <React.Fragment>
      <MetaSec manuscript={manuscript} />
      <Checklist>
        <Col flex={1}>
          <div>Submitted</div>
          <Status current={manuscript.status} mark="READY" />
        </Col>
        {!preprint && (
          <Col flex={1.75}>
            <div>Author review</div>
            <Status current={manuscript.status} mark="in-review" />
          </Col>
        )}
        <Col flex={1.75}>
          <div>Submission QA</div>
          <Status current={manuscript.status} mark="submitted" />
        </Col>
        <Col flex={1.5}>
          <div>XML tagging</div>
          <Status current={manuscript.status} mark="tagging" />
        </Col>
        <Col flex={1}>
          <div>XML QA</div>
          <Status
            current={manuscript.status}
            mark={
              (indexOnly && manuscript.status) ||
              (manuscript.status === 'xml-triage' && 'xml-triage') ||
              (manuscript.status === 'file-error' && 'file-error') ||
              'xml-qa'
            }
          />
        </Col>
        <Col flex={1.5}>
          <div>Final review</div>
          <Status
            current={manuscript.status}
            mark={indexOnly ? 'repo-processing' : 'xml-review'}
          />
        </Col>
        <Col flex={1}>
          <div>Citation</div>
          <CheckCitation manuscript={manuscript} />
        </Col>
        <Col flex={preprint ? 1 : 0.5}>
          <div>Sent</div>
          <Status
            current={manuscript.status}
            mark={
              (indexOnly && 'repo-processing') ||
              (manuscript.status === 'repo-triage' && 'repo-triage') ||
              'repo-ready'
            }
          />
        </Col>
        <Col flex={1}>
          <div>Published</div>
          <Status current={manuscript.status} mark="repo-processing" />
        </Col>
      </Checklist>
    </React.Fragment>
  )
}

export default QuickView
