import React from 'react'
import { ApolloConsumer } from 'react-apollo'
import styled from 'styled-components'
import moment from 'moment'
import { th } from '@pubsweet/ui-toolkit'
import { Action, Button, H3, H4, Link } from '@pubsweet/ui'
import {
  Buttons,
  Loading,
  LoadingIcon,
  Notification,
  SearchForm,
  Toggle,
  ZebraList,
  NoClick,
} from '../ui'
import { JOURNAL_INFO } from './operations'
import PubMedSearchResult from './PubMedSearchResult'
import UnmatchedCitation from './UnmatchedCitation'
import FullTextChecks from './FullTextChecks'

const LoadMore = styled(Button)`
  margin: 15px auto;
`
const Notice = styled(H4)`
  margin: 0 auto;
  & + div {
    margin-bottom: calc(${th('gridUnit')} * 3);
  }
`
const FlexP = styled.p`
  display: flex;
  align-items: center;
  & > button {
    flex: 0 0 auto;
  }
  & > span {
    flex: 1 1 50%;
    margin-left: ${th('gridUnit')};
  }
`
const isDate = d => {
  const obj = {
    jatsDate: {},
  }
  let date = moment.utc(d, 'YYYY MMM DD')
  if (date.isValid()) {
    obj.jatsDate.day = moment(date).format('DD')
  } else {
    date = moment.utc(d, 'YYYY MMM')
    if (!date.isValid) {
      return false
    }
  }
  obj.jatsDate.year = moment(date).format('YYYY')
  obj.jatsDate.month = moment(date).format('MM')
  obj.date = date
  return obj
}
class PubMedSearch extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      query: this.props.search || '',
      retstart: 0,
      results: [],
      enabled: !!this.props.search,
      loading: false,
      hitcount: null,
      inPMC: null,
      inEPMC: false,
      unmatched: false,
      cannotLoad: false,
      submitting: false,
    }
    this.onSearch = this.onSearch.bind(this)
    this.onLoad = this.onLoad.bind(this)
    this.onQueryChange = this.onQueryChange.bind(this)
    this.onSelected = this.onSelected.bind(this)
    this.selectResult = this.selectResult.bind(this)
    this.handleUnmatched = this.handleUnmatched.bind(this)
  }
  componentDidMount() {
    if (
      this.props.manuscript &&
      (!this.props.manuscript.meta.articleIds ||
        this.props.manuscript.meta.articleIds.length === 0)
    ) {
      this.setState({ unmatched: true })
    }
    if (this.props.search) {
      this.onSearch()
    }
  }
  componentDidUpdate(prevProps, prevState) {
    const { inPMC, inEPMC } = this.state
    if ((inPMC && !prevState.inPMC) || (inEPMC && !prevState.inEPMC)) {
      this.props.changeInfo({
        cancelCheck: !!inPMC,
        inEPMC: inEPMC && inPMC.pmcid,
      })
    }
    if (!inPMC && prevState.inPMC) {
      this.props.changeInfo({ cancelCheck: null })
    }
  }
  handleUnmatched(unm) {
    const citationData = unm
    if (this.props.newNote && unm.note) {
      const { notes } = this.props.manuscript.meta
      const note = notes
        ? notes.find(n => n.notesType === 'userCitation')
        : null
      if (note) {
        this.props.changeNote({ id: note.id, ...unm.note })
      } else {
        this.props.newNote({
          manuscriptId: this.props.manuscript.id,
          ...unm.note,
        })
      }
      delete citationData.note
    }
    this.props.citationData(citationData)
  }
  selectResult(result, journal, removePMCID) {
    const publicationDates = []
    const articleIds = [
      {
        pubIdType: 'pmid',
        id: result.uid,
      },
    ]
    if (result.pubdate) {
      const ppub = { type: 'ppub' }
      const pIsDate = isDate(result.pubdate)
      if (pIsDate) {
        ppub.date = pIsDate.date
        ppub.jatsDate = pIsDate.jatsDate
      } else {
        ppub.date = moment.utc(result.pubdate.substring(0, 4), 'YYYY')
        ppub.jatsDate = {
          year: result.pubdate.substring(0, 4),
          season: result.pubdate.substring(5),
        }
      }
      publicationDates.push(ppub)
    }
    if (result.epubdate) {
      const epub = { type: 'epub' }
      const eIsDate = isDate(result.epubdate)
      if (eIsDate) {
        epub.date = eIsDate.date
        epub.jatsDate = eIsDate.jatsDate
      } else {
        epub.date = moment.utc(result.epubdate.substring(0, 4), 'YYYY')
        epub.jatsDate = {
          year: result.epubdate.substring(0, 4),
          season: result.epubdate.substring(5),
        }
      }
      publicationDates.push(epub)
    }
    if (result.articleids) {
      const doi = result.articleids.find(i => i.idtype === 'doi')
      const pmc = result.articleids.find(i => i.idtype === 'pmc')
      doi &&
        articleIds.push({
          pubIdType: 'doi',
          id: doi.value,
        })
      pmc &&
        articleIds.push({
          pubIdType: 'pmcid',
          id: pmc.value,
        })
    }
    const pages = result.pages && result.pages.split('-')
    const citationData = {
      journalId: journal.id,
      meta: {
        unmatchedJournal: null,
        title: result.title,
        volume: result.volume,
        issue: result.issue,
        location: {
          fpage: pages ? pages[0] : null,
          lpage: pages ? pages.pop() : null,
          elocationId: result.elocationid,
        },
        publicationDates,
        articleIds,
      },
    }
    this.props.citationData(citationData, removePMCID)
  }
  onQueryChange(event) {
    this.setState({
      enabled: event ? !!event.target.value : false,
      retstart: 0,
      results: [],
      hitcount: null,
      cannotLoad: false,
      query: event ? event.target.value : '',
    })
  }
  onLoad(event) {
    this.onSearch(event)
  }
  async onSearch(event) {
    event && event.preventDefault()
    this.setState({
      enabled: false,
      loading: true,
    })
    const { query, retstart, results } = this.state

    if (retstart >= results.length && query) {
      const SearchUrl = `/eutils/esearch?term=${query
        .replace(/http.*doi\.org\//gi, '')
        .replace(/(.*)\W$/g, '$1')}&retstart=${retstart}&db=pubmed`

      try {
        const response = await fetch(SearchUrl, {
          headers: new Headers({
            Authorization: `Bearer ${window.localStorage.getItem('token')}`,
          }),
        })
        const json = await response.json()
        const ids = json.esearchresult.idlist
        const hitcount = parseInt(json.esearchresult.count, 10)
        const SummaryUrl = `/eutils/esummary?db=pubmed&id=${ids.join()}`
        const summary = await fetch(SummaryUrl, {
          headers: new Headers({
            Authorization: `Bearer ${window.localStorage.getItem('token')}`,
          }),
        })
        const summaryResponse = await summary.json()
        const { result } = summaryResponse
        const newResults = this.state.results.slice()
        ids.forEach(id => {
          newResults.push(result[id])
        })
        this.setState({
          hitcount,
          results: newResults,
          retstart: newResults.length > 0 ? newResults.length : 0,
        })
      } catch (e) {
        this.setState({
          cannotLoad: true,
          loading: false,
        })
      }
    }
    this.setState({ loading: false })
  }
  async onSelected(result, journal) {
    const pmcid =
      result.articleids.find(id => id.idtype === 'pmc') &&
      result.articleids.find(id => id.idtype === 'pmc').value
    // Embargo check
    const releaseDate =
      result.history.find(d => d.pubstatus === 'pmc-release') &&
      result.history.find(d => d.pubstatus === 'pmc-release').date
    const rDate = releaseDate && moment.utc(releaseDate, 'YYYY/MM/DD kk:mm')
    const pDate = isDate(result.pubdate) && isDate(result.pubdate).date
    const eDate = isDate(result.epubdate) && isDate(result.epubdate).date
    // If the article has a release date that is not a pubdate it has an embargo
    const embargo = rDate && !rDate.isSame(pDate) && !rDate.isSame(eDate)
    if (pmcid) {
      this.setState({ loading: true })
      const obj = {
        loading: false,
        submitting: false,
      }
      // License check
      const epmcURL = `https://www.ebi.ac.uk/europepmc/webservices/rest/search?query=SRC:MED%20EXT_ID:${result.uid}&resulttype=core&format=json`
      const response = await fetch(epmcURL)
      const json = await response.json()
      // On initial search only stop submissions for CC BY and 0 embargo
      const { license } = json.resultList.result[0] || {}
      const planSReq = !!(license === 'cc by' && !embargo)
      if (planSReq || this.props.lenientChecks) {
        obj.inPMC = { pmcid, result, journal, planSReq }
        this.setState(obj)
      } else {
        this.selectResult(result, journal)
      }
    } else {
      this.selectResult(result, journal)
    }
  }
  render() {
    const {
      results,
      hitcount,
      loading,
      query,
      inPMC,
      unmatched,
      cannotLoad,
      submitting,
    } = this.state
    const {
      manuscript,
      toggle,
      adminRemove,
      lenientChecks,
      endSubmission: EndSub,
    } = this.props
    const { meta, journal } = manuscript || {}
    const { notes, articleIds } = meta || {}
    const note = notes ? notes.find(n => n.notesType === 'userCitation') : null
    const doi = articleIds ? articleIds.find(a => a.pubIdType === 'doi') : null
    let journalInfo = journal
    if (!journalInfo && meta && meta.unmatchedJournal) {
      journalInfo = {
        journalTitle: meta.unmatchedJournal,
      }
    }
    return (
      <React.Fragment>
        {submitting && <NoClick />}
        {toggle && (
          <Toggle>
            <Action
              className={!unmatched && 'current'}
              onClick={() => this.setState({ unmatched: false })}
            >
              Populate from PubMed
            </Action>
            <Action
              className={unmatched && 'current'}
              onClick={() => this.setState({ unmatched: true })}
            >
              Enter manually
            </Action>
          </Toggle>
        )}
        {unmatched ? (
          <UnmatchedCitation
            citation={unm => {
              this.setState({ submitting: true }, () =>
                this.handleUnmatched(unm),
              )
            }}
            close={!toggle ? () => this.setState({ unmatched: false }) : false}
            doi={doi ? doi.id : ''}
            journal={journalInfo}
            note={note ? note.content : ''}
            title={meta && meta.title}
          />
        ) : (
          <React.Fragment>
            {inPMC ? (
              <React.Fragment>
                <FullTextChecks
                  inEPMC={v => this.setState({ inEPMC: v })}
                  journal={lenientChecks ? inPMC.journal : null}
                  pmcid={inPMC.pmcid}
                  pmid={inPMC.result.uid}
                  title={inPMC.result.title}
                />
                {adminRemove && (
                  <React.Fragment>
                    <H4>Admin options</H4>
                    {!inPMC.planSReq && (
                      <Notification type="warning">
                        Check that grants and Plan S status are correct before
                        canceling submission!
                      </Notification>
                    )}
                    <FlexP>
                      <Button
                        onClick={() =>
                          this.setState({ submitting: true }, () =>
                            this.selectResult(inPMC.result, inPMC.journal),
                          )
                        }
                      >
                        Use citation data
                      </Button>
                      <span>
                        Force use of this citation data regardless of manuscript
                        being already in PMC.
                      </span>
                    </FlexP>
                    <FlexP>
                      <Button
                        onClick={() => {
                          this.setState({ submitting: true }, () =>
                            this.selectResult(
                              inPMC.result,
                              inPMC.journal,
                              inPMC.pmcid,
                            ),
                          )
                        }}
                      >
                        Remove submission
                      </Button>
                      <span>
                        Cancel the submission as already in PMC. Any associated
                        grants will be linked, and an email will be sent to the
                        submitter.
                      </span>
                    </FlexP>
                  </React.Fragment>
                )}
                <Buttons>
                  {EndSub ? (
                    <EndSub />
                  ) : (
                    <Link to="/">
                      <Button primary>End submission</Button>
                    </Link>
                  )}
                  <Button onClick={() => this.setState({ inPMC: null })}>
                    Back to Search
                  </Button>
                </Buttons>
              </React.Fragment>
            ) : (
              <React.Fragment>
                <H3>Populate from PubMed</H3>
                <SearchForm
                  buttonLabel="Search"
                  disabled={!this.state.enabled}
                  label="Search by article title, PMID, DOI, etc."
                  name="Search"
                  onChange={this.onQueryChange}
                  onSubmit={this.onSearch}
                  placeholder="Search for your article"
                  value={query}
                />
                {cannotLoad && (
                  <Notification type="warning">
                    {`Cannot load results. Please try a more specific query. Or, `}
                    <Action onClick={() => this.setState({ unmatched: true })}>
                      enter your citation manually
                    </Action>
                    .
                  </Notification>
                )}
                {hitcount !== null && (
                  <React.Fragment>
                    <Notice>Select your citation from results</Notice>
                    {(hitcount === 0 || results.length > 0) && (
                      <React.Fragment>
                        {this.props.changeNote ? (
                          <Notification type="info">
                            {hitcount === 0 && 'No results found. '}
                            {results.length > 0 &&
                              'Manuscript not in results? '}
                            <Action
                              onClick={() => this.setState({ unmatched: true })}
                            >
                              Click to enter citation manually.
                            </Action>
                          </Notification>
                        ) : (
                          <React.Fragment>
                            {hitcount === 0 && (
                              <Notification type="info">
                                No results found.
                              </Notification>
                            )}
                          </React.Fragment>
                        )}
                      </React.Fragment>
                    )}
                  </React.Fragment>
                )}
                {results.length > 0 && (
                  <ApolloConsumer>
                    {client => (
                      <ZebraList style={{ textAlign: 'center' }}>
                        {results.map(
                          result =>
                            result.fulljournalname && (
                              <PubMedSearchResult
                                key={result.uid}
                                onClick={async () => {
                                  const { data } = await client.query({
                                    query: JOURNAL_INFO,
                                    variables: { nlmId: result.nlmuniqueid },
                                  })
                                  this.setState({ submitting: true }, () =>
                                    this.onSelected(result, data.selectWithNLM),
                                  )
                                }}
                                result={result}
                              />
                            ),
                        )}
                        {loading && (
                          <Loading>
                            <LoadingIcon />
                          </Loading>
                        )}
                        {results.length < hitcount && !loading && (
                          <LoadMore onClick={this.onLoad} secondary>
                            Load More Results
                          </LoadMore>
                        )}
                      </ZebraList>
                    )}
                  </ApolloConsumer>
                )}
                {results.length === 0 && loading && (
                  <Loading>
                    <LoadingIcon />
                  </Loading>
                )}
              </React.Fragment>
            )}
          </React.Fragment>
        )}
      </React.Fragment>
    )
  }
}

export default PubMedSearch
