const logger = require('@pubsweet/logger')
const express = require('express')
const { ManuscriptManager, FileManager } = require('../xpub-model')
const getUser = require('../utils/user.js')
const { addCitation } = require('../xml-validation')
const { states } = require('config')

module.exports = async app => {
  const adminUser = await getUser.getAdminUser()
  app.use(express.json())
  // wait to prevent posting and receiving callback in same second
  app.post('/api/id/', async (req, res) => {
    // accept author manuscript callback
    const body = await req.body
    const { clientId } = body
    if (clientId) {
      if (ManuscriptManager.findById(clientId, adminUser.id, null, false)) {
        res.status(200).send('Callback response recieved')
        await new Promise(resolve => setTimeout(resolve, 1000))
        completeCitation(body, req.query.token)
      } else {
        res.status(503).send('ERROR: Database unavailable')
      }
    } else {
      res.status(400).send('ERROR: No clientId included in request body')
    }
  })
  app.post('/api/pprid/', async (req, res) => {
    // accept preprint callback
    const body = await req.body
    const { clientId } = body
    if (clientId) {
      if (ManuscriptManager.findById(clientId, adminUser.id, null, false)) {
        const { token, indexOnly } = req.query
        res.status(200).send('Callback response recieved')
        await new Promise(resolve => setTimeout(resolve, 1000))
        preprintComplete(body, token, !!indexOnly)
      } else {
        res.status(503).send('ERROR: Database unavailable')
      }
    } else {
      res.status(400).send('ERROR: No clientId included in request body')
    }
  })
  app.post('/api/idupdate/', async (req, res) => {
    // accept callback without changing manuscript status
    const body = await req.body
    const { clientId } = body
    if (clientId) {
      if (ManuscriptManager.findById(clientId, adminUser.id, null, false)) {
        const { token, indexOnly } = req.query
        res.status(200).send('Callback response recieved')
        await new Promise(resolve => setTimeout(resolve, 1000))
        resendComplete(body, token, !!indexOnly)
      } else {
        res.status(503).send('ERROR: Database unavailable')
      }
    } else {
      res.status(400).send('ERROR: No clientId included in request body')
    }
  })
  app.post('/api/delete/', async (req, res) => {
    // accept deletion callback
    const body = await req.body
    const { clientId } = body
    if (clientId) {
      if (ManuscriptManager.findById(clientId, adminUser.id, null, false)) {
        const { token, replace } = req.query
        res.status(200).send('Callback response recieved')
        await new Promise(resolve => setTimeout(resolve, 900))
        withdrawCitation(body, token, !!replace)
      } else {
        res.status(503).send('ERROR: Database unavailable')
      }
    } else {
      res.status(400).send('ERROR: No clientId included in request body')
    }
  })
}

async function completeCitation(data, token) {
  const { clientId, id, message, success } = data
  logger.info(`Processing EBI response for ${clientId}`)
  const adminUser = await getUser.getAdminUser()
  try {
    const manuscript = await ManuscriptManager.findById(clientId, adminUser.id)
    if (success && id) {
      // check token validity
      const xmlFile = await FileManager.findByManIdAndType(clientId, 'PMCfinal')
      if (!xmlFile[0] || xmlFile[0].id !== token) {
        throw new Error('Bad or out of date XML file token, please resend')
      }
      const { articleIds } = manuscript.meta
      const hasId = articleIds.some(aid => aid.pubIdType === 'pmcid')
      const meta = {
        articleIds: articleIds.filter(aid => aid.pubIdType !== 'pmcid'),
      }
      meta.articleIds.push({
        pubIdType: 'pmcid',
        id,
      })
      if (!hasId) {
        meta.articleIds.push({
          pubIdType: 'pre-pmc',
          id,
        })
        const { manuscript: dupeMan } = await ManuscriptManager.findByArticleId(
          id,
          adminUser.id,
        )
        if (dupeMan) {
          throw new Error(
            `EBI tried to assign ${id}. ${id} already assigned to ${dupeMan.id}. Please contact EBI fulltext repository.`,
          )
        }
      }
      logger.info(`Success. ${clientId} received by EBI`)
      await ManuscriptManager.update(
        {
          formState: null,
          id: clientId,
          meta,
          decision: 'accepted',
          ebiState: hasId ? 'EBI received' : 'adding PMCID to EBI package',
          status: hasId ? 'repo-processing' : manuscript.status,
        },
        adminUser.id,
      )
      if (!hasId) {
        await new Promise(resolve => setTimeout(resolve, 1000))
        await addCitation(clientId)
      }
    } else {
      if (!id) {
        throw new Error(`No PMCID generated. ${message}`)
      }
      throw new Error(message)
    }
  } catch (err) {
    await ManuscriptManager.update(
      {
        id: clientId,
        formState: `Error from EBI: ${err.message}`,
        status: 'repo-triage',
        pdfDepositState: null,
      },
      adminUser.id,
    )
    logger.error(err)
  }
}

async function preprintComplete(data, token, indexOnly) {
  const { clientId, message, success } = data
  logger.info(`Processing EBI response for ${clientId}`)
  const adminUser = await getUser.getAdminUser()
  try {
    if (success) {
      // check token validity
      const xmlFile = await FileManager.findByManIdAndType(clientId, 'PMCfinal')
      if (!xmlFile[0] || xmlFile[0].id !== token) {
        throw new Error('Bad or out of date XML file token, please resend')
      }
      logger.info('Full text successfully registered at EBI')
      const updateObj = {
        formState: null,
        id: clientId,
        decision: 'accepted',
        status: indexOnly ? 'xml-qa' : 'repo-processing',
        ebiState: indexOnly ? 'EBI received index only copy' : 'success',
      }
      await ManuscriptManager.update(updateObj, adminUser.id)
    } else {
      throw new Error(message)
    }
  } catch (err) {
    await ManuscriptManager.update(
      {
        id: clientId,
        formState: `Error from EBI: ${err.message}`,
        status: 'repo-triage',
        ebiState: indexOnly ? 'index only' : null,
        pdfDepositState: null,
      },
      adminUser.id,
    )
    logger.error(err)
  }
}

async function resendComplete(data, token, indexOnly) {
  const { clientId, id, message, success } = data
  logger.info(`Processing EBI response for ${clientId}`)
  const adminUser = await getUser.getAdminUser()
  try {
    if (success && id) {
      // check token validity
      const xmlFile = await FileManager.findByManIdAndType(clientId, 'PMCfinal')
      if (!xmlFile[0] || xmlFile[0].id !== token) {
        throw new Error('Bad or out of date XML file token, please resend')
      }
      logger.info('Full text successfully registered at EBI')
      const updateObj = {
        id: clientId,
        decision: 'accepted',
        ebiState:
          (id.startsWith('PMC') && 'EBI received') ||
          (indexOnly && 'EBI received index only copy') ||
          'success',
      }
      await ManuscriptManager.update(updateObj, adminUser.id)
    } else {
      throw new Error(message)
    }
  } catch (err) {
    await ManuscriptManager.update(
      {
        id: clientId,
        formState: `Error from EBI: ${err.message}`,
        status: 'repo-triage',
        ebiState: indexOnly ? 'index only' : null,
        pdfDepositState: null,
      },
      adminUser.id,
    )
    logger.error(err)
  }
}

async function withdrawCitation(data, token, replace) {
  const { clientId, id, message, success } = data
  logger.info(`Processing EBI response for ${clientId}`)
  const adminUser = await getUser.getAdminUser()
  try {
    if (success) {
      // check token validity
      const xmlFile = await FileManager.find(token)
      if (!xmlFile || xmlFile.manuscriptId !== clientId) {
        throw new Error(
          'Bad or out of date XML file token, please attempt withdrawal again',
        )
      }
      if (replace) {
        const upMan = await ManuscriptManager.update(
          {
            id: clientId,
            ebiState: 'Record removed',
            decision: null,
          },
          adminUser.id,
        )
        if (states.indexOf(upMan.status) >= states.indexOf('repo-ready')) {
          await ManuscriptManager.update(
            { id: clientId, status: 'repo-ready' },
            adminUser.id,
          )
          await addCitation(clientId)
        }
      } else if (id.startsWith('PMC')) {
        await ManuscriptManager.update(
          {
            id: clientId,
            ebiState: 'Record removed',
            ncbiState: 'ncbi-withdraw',
          },
          adminUser.id,
        )
      } else if (id.startsWith('PPR')) {
        await ManuscriptManager.delete(clientId, adminUser.id)
      } else {
        throw new Error('Unknown ID type')
      }
    } else {
      throw new Error(message)
    }
  } catch (err) {
    await ManuscriptManager.update(
      {
        id: clientId,
        formState: `Withdrawal error from EBI: ${err.message}`,
        status: 'withdrawal-triage',
        pdfDepositState: null,
      },
      adminUser.id,
    )
    logger.error(err)
  }
}
