require('dotenv').config()
const htmlDecode = require('decode-html')
const uuidv4 = require('uuid/v4')
const logger = require('@pubsweet/logger')
const lodash = require('lodash')
const mime = require('mime-types')
const fs = require('fs')
const readline = require('readline')
const chokidar = require('chokidar')
const xpath = require('xpath')
const Dom = require('xmldom').DOMParser
const path = require('path')
const { exec } = require('child_process')
const config = require('config')
const moment = require('moment')
const async = require('async')
const { transaction } = require('objection')
const {
  ManuscriptManager,
  NoteManager,
  FileManager,
  TeamManager,
  OrganizationManager,
} = require('../xpub-model/')
const Journal = require('../xpub-model/entities/journal/data-access')
const Identity = require('../xpub-model/entities/identity/data-access')
const fileUtils = require('../utils/files')
const ftpUtils = require('../utils/ebi-ftp.js')
const { createTempDir } = require('../utils/unTar')
const { xsweetConvert } = require('../xsweet-conversion')
const { bulkUploaderEmail, errorDevEmail } = require('../email')

global.token = ''

// Initialize watcher.
const rootPath = `${process.env.HOME}/${config.get('ftp_directory')}/`
const ignoreTaggerFolder = `${rootPath}${config.get('ftp_tagger').username}/*`
const { internalServiceUrl } = config.get('pubsweet-server')
const { states } = config

const watcher = chokidar.watch(`${rootPath}**/*.tar.gz`, {
  persistent: true,
  usePolling: true,
  ignored: ignoreTaggerFolder,
  awaitWriteFinish: {
    stabilityThreshold: 10000, // 60000
    pollInterval: 1000, // 2000
  },
})

const tidyUp = async (filePath, tmpPath, manId, isError) => {
  try {
    const emsid = manId || 'UNKNOWN'
    let parentDir = filePath

    // Issue 162
    // go up directories until you find the ftp root path. This is where the dated folder should be created.
    do {
      parentDir = path.dirname(parentDir)
    } while (`${path.dirname(parentDir)}/` !== rootPath)

    const fileName = path.basename(filePath)
    const fileNameExt = isError ? 'ERROR.' : `loaded.mid-${emsid}.`
    const currentDate = moment(new Date()).format('YYYY-MM-DD')
    const dateExt = moment(new Date()).format('YYYY-MM-DD_HH:mm:ss')
    const datedFolder = `${parentDir}/${currentDate}`
    const updatedFilepath = `${datedFolder}/${fileName}.${fileNameExt}${dateExt}`

    const cmd = `mkdir -p ${datedFolder}; mv ${filePath} ${updatedFilepath} ; rm -rf ${tmpPath}`
    return new Promise((resolve, reject) => {
      exec(cmd, (err, stdout, stderr) => {
        if (err) {
          // node couldn't execute the command
          reject(err)
        }
        if (isError) {
          logger.info(`Error in Bulk import process for ${filePath}`)
        } else {
          logger.info(
            `Finished Bulk import process for ${filePath} as manuscript ${manId}.`,
          )
        }
        resolve(updatedFilepath)
      })
    })
  } catch (e) {
    logger.error('Error while tidying up after another error!')
  }
  return Journal.knex().destroy()
}

const queryIdentity = async path => {
  const rootParts = rootPath.split('/')
  const fileNameParts = path.split('/')
  const username = fileNameParts[rootParts.length - 1]
  const trx = await transaction.start(Identity.knex())
  try {
    const user = await Identity.findByFtpUsername(username, trx)
    trx.commit()
    return user
  } catch (err) {
    if (trx) trx.rollback()
    logger.error(`Cannot access database; sending ${path} back to queue`)
    await new Promise(resolve => setTimeout(resolve, 5000))
    queue.push(path)
    return true
  }
}

const queue = async.queue(async taskPath => {
  const submitter = await queryIdentity(taskPath)
  if (submitter) {
    logger.info(`Submitter Details: ${submitter}`)
    logger.info(`Initialising job for file: ${taskPath}`)
    await runProcess(taskPath, submitter)
    logger.info(`Job complete for file: ${taskPath}`)
    return true
  }
  logger.error(`Submitter with details ${submitter} not found`)
})

queue.drain(() => {
  logger.info('All files processed.')
})

queue.error((err, task) => {
  logger.error(`Error in task ${task}: ${err}`)
})

process
  .on('uncaughtException', (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    tidyUp(err.filePath, err.tmpPath, err.manId, true)
  })
  .on('unhandledRejection', (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    tidyUp(reason.filePath, reason.tmpPath, reason.manId, true)
  })

watcher
  .on('add', path => {
    logger.info(`New file ${path} has been detected`)
    queue.push(path)
  })
  .on('change', path => logger.debug(`File ${path} has been changed`))
  .on('unlink', path => logger.debug(`File ${path} has been removed`))

// More possible events.
watcher
  .on('addDir', path => logger.debug(`Directory ${path} has been added`))
  .on('unlinkDir', path => logger.debug(`Directory ${path} has been removed`))
  .on('error', error => {
    logger.error(`Watcher error: ${error}`)
    errorDevEmail('the FTP utility', `Watcher error: ${error}`)
  })
  .on('ready', () =>
    logger.info(
      `Initial scan complete. Ready for changes under folder ${rootPath}`,
    ),
  )
  .on('raw', (event, path, details) => {
    logger.debug('Raw event info:', event, path, details)
  })

// 'add', 'addDir' and 'change' events also receive stat() results as second
// argument when available: http://nodejs.org/api/fs.html#fs_class_fs_stats
watcher.on('change', (path, stats) => {
  if (stats) logger.debug(`File ${path} changed size to ${stats.size}`)
})

const runProcess = async (packagePath, submitter) => {
  let packageName
  let tmpPath
  let manuscriptId
  let suffixedFilePath
  try {
    logger.info(`Starting Bulk Import Process for ${packagePath}`)
    packageName = path.basename(packagePath)
    tmpPath = await createTempDir()
    const tmpPathExt = await extractFiles(packagePath, tmpPath)
    // rename file - append .start suffix
    suffixedFilePath = await fileUtils.renameFileSuffix(packagePath)
    const parsedInfo = await parseManifest(tmpPathExt, packageName)

    const xmlObj = await parseXml(parsedInfo.metaXml, packageName)
    manuscriptId = await createManuscript(
      xmlObj,
      parsedInfo,
      tmpPath,
      submitter,
      packageName,
      suffixedFilePath,
    )
    const updatedFilepath = await tidyUp(
      suffixedFilePath,
      tmpPath,
      manuscriptId,
    )
    await updateFTP(updatedFilepath, submitter)
    return true
  } catch (error) {
    if (submitter) {
      bulkUploaderEmail(submitter.email, error, packageName)
    }
    errorDevEmail(
      'the FTP utility',
      `Error with ftp-integration - bulk upload<br/>Package: ${packageName}<br/>${error}`,
    )
    logger.error('Error with ftp-integration - api.js', error.stack)
    const updatedFilepath = await tidyUp(
      suffixedFilePath,
      tmpPath,
      manuscriptId,
      true,
    )
    await updateFTP(updatedFilepath, submitter)
    return true
  }
}

const extractFiles = (source, dest) =>
  new Promise((resolve, reject) => {
    const cmd = `tar vzxf ${source} -C ${dest} --xform='s#^.+/##x'` // the xform removes all directories
    exec(cmd, err => {
      if (err) {
        logger.error('Error extracting FTP bulk upload files')
        // node couldn't execute the command
        reject(err)
      }
      resolve(dest)
    })
  })

const parseManifest = async (tempFolder, packageName) => {
  const files = []
  let metaXml = ''
  logger.info(`${packageName}: Parsing package manifest.`)
  const lineReader = readline.createInterface({
    input: fs.createReadStream(`${tempFolder}/manifest.txt`, {
      encoding: 'UTF-8',
    }),
  })

  const manifestList = await new Promise((resolve, reject) => {
    const lines = []
    lineReader
      .on('line', line => {
        if (/\S/.test(line)) {
          const row = line.trim().split(/\t{1,}/)
          const filename = row.pop()
          const [type, label] = row
          lines.push({ type, label, filename })
        }
      })
      .on('close', () => {
        resolve(lines)
      })
  })

  if (!manifestList.some(f => f.type === 'manuscript')) {
    throw new Error(`The package does not contain a manuscript file.`)
  }

  await manifestList.reduce(async (promise, file, i, mL) => {
    await promise
    if (file.type === 'manuscript') {
      if (mL.findIndex(f => f.type === 'manuscript') < i) {
        file.type = 'manuscript_secondary'
      }
    }
    const { type, label, filename } = file
    if (!filename || !fs.existsSync(`${tempFolder}/${filename}`)) {
      throw new Error(`Listed file ${filename || ''} is not in the package.`)
    }
    if (type === 'supplement' && label.length > 100) {
      throw new Error(
        `The label provided for file ${filename} exceeds the character limit. Please limit labels to 100 characters.`,
      )
    }
    if (type === 'bulksub_meta_xml') {
      metaXml = `${tempFolder}/${filename}`
    } else {
      files.push({
        fileURI: `${tempFolder}/${filename}`,
        filename,
        type,
        info: label,
      })
    }
    return Promise.resolve()
  }, Promise.resolve())

  return { metaXml, files }
}

const parseXml = (xmlFile, packageName) =>
  new Promise((resolve, reject) => {
    logger.info(`${packageName}: Parsing package metadata XML`)
    fs.readFile(xmlFile, 'utf8', (err, data) => {
      if (err) {
        logger.error(`${packageName}: Cannot read XML file`)
        reject(err)
      }

      const doc = new Dom().parseFromString(data)
      const nodeset = xpath.select1('/nihms-submit', doc)
        ? xpath.select1('/nihms-submit/manuscript', doc)
        : xpath.select1('/manuscript-submit', doc)

      const manuscript_embargo_attr =
        xpath.select1("@*[starts-with(name(), 'embargo')]", nodeset) &&
        xpath.select1("@*[starts-with(name(), 'embargo')]", nodeset).value
          ? xpath.select1("@*[starts-with(name(), 'embargo')]", nodeset).value
          : 0
      const permissions = xpath.select1('//permissions', doc)
        ? xpath
            .select1('//permissions', doc)
            .toString()
            .replace(/&lt;/g, '<')
        : null
      const journal_meta_nodeset = xpath.select1('//journal-meta', doc)
      const journal_title_val = xpath.select1(
        'journal-title',
        journal_meta_nodeset,
      ).firstChild.data

      let journal_nlmta = null
      try {
        if (xpath.select1('/nihms-submit', doc)) {
          journal_nlmta = xpath.select1(
            'journal-id[@journal-id-type="nlm-ta"]',
            journal_meta_nodeset,
          ).firstChild.data
        } else {
          journal_nlmta = xpath.select1('nlm-ta', journal_meta_nodeset)
            .firstChild.data
        }
      } catch (ignore) {
        logger.debug('Could not find nlmta')
      }

      let reviewers = []
      xpath.select('//contacts/person', doc).forEach(personElement => {
        const email = xpath.select1('@email', personElement).value
        const corrpi =
          xpath.select1('@corrpi="yes"', personElement) ||
          xpath.select1('@person-type="reviewer"', personElement)
        if (email && !reviewers.find(reviewer => reviewer.email === email)) {
          reviewers.push({
            givenNames: `${xpath.select1('@fname', personElement).value}`,
            surname: `${xpath.select1('@lname', personElement).value}`,
            email,
            corrpi,
          })
        }
      })
      if (reviewers.length === 0) reviewers = null

      let versionList = []
      xpath.select('//versionList/pprid', doc).forEach(pprid => {
        versionList.push(pprid.firstChild.data)
      })
      if (versionList.length === 0) versionList = null

      // title nodeset
      const title_value = lodash.escape(
        xpath.select1('/nihms-submit/title', doc)
          ? xpath.select1('/nihms-submit/title', doc).firstChild.data
          : xpath.select1('/manuscript-submit/manuscript-title', doc).firstChild
              .data,
      )

      let preprint = false
      let publisherPDF = false
      let pprid = null
      let version = 0
      const articleIds = []
      const publicationDates = []

      if (xpath.select1('@pprid', nodeset)) {
        preprint = true
        if (xpath.select1('@pprid', nodeset).value) {
          pprid = xpath.select1('@pprid', nodeset).value
          articleIds.push({
            pubIdType: 'pprid',
            id: pprid,
          })
        }
        if (
          xpath.select1('@version', nodeset) &&
          xpath.select1('@version', nodeset).value
        ) {
          version = xpath.select1('@version', nodeset).value
        }
        if (xpath.select1('@version-date', nodeset)) {
          const date = xpath.select1('@version-date', nodeset).value
          if (date) {
            const jatsDate = date.split('-')
            publicationDates.push({
              type: 'epub',
              date: `${date}T00:00:00+00:00`,
              jatsDate: {
                year: jatsDate[0],
                month: jatsDate[1],
                day: jatsDate[2],
              },
            })
          }
        }
      }

      if (
        xpath.select1('@pmid', nodeset) &&
        xpath.select1('@pmid', nodeset).value
      ) {
        articleIds.push({
          pubIdType: 'pmid',
          id: xpath.select1('@pmid', nodeset).value,
        })
      }
      if (
        xpath.select1('@doi', nodeset) &&
        xpath.select1('@doi', nodeset).value
      ) {
        articleIds.push({
          pubIdType: 'doi',
          id: xpath.select1('@doi', nodeset).value,
        })
      }

      if (
        xpath.select1('@show_publisher_pdf', nodeset) &&
        xpath.select1('@show_publisher_pdf', nodeset).value &&
        xpath.select1('@show_publisher_pdf', nodeset).value === 'yes'
      ) {
        publisherPDF = true
      }

      const xmlObj = {
        title: htmlDecode(title_value),
        articleIds,
        journal: journal_title_val,
        journal_nlmta,
        releaseDelay: manuscript_embargo_attr,
        reviewers,
        permissions,
        publicationDates,
        publisherPDF,
        preprint,
        version,
        pprid,
        versionList,
      }
      resolve(xmlObj)
    })
  })

const createManuscript = async (
  xmlObj,
  parsedInfo,
  tmpPath,
  submitter,
  packageName,
  packagePath,
) => {
  const {
    title,
    releaseDelay,
    articleIds,
    publicationDates,
    permissions,
    reviewers,
    journal_nlmta,
    journal,
    publisherPDF,
    preprint,
    version,
    pprid,
    versionList,
  } = xmlObj
  const { files } = parsedInfo
  const manuscript = {
    version,
    meta: {
      title,
      releaseDelay,
      articleIds,
      publicationDates,
    },
  }
  const trx = await transaction.start(Journal.knex())
  try {
    // audrey (20190122): If there is not a single NLM TA match you should have the journal as unmatchedJournal
    // and let the helpdesk figure it out
    const journalSearch = journal_nlmta
      ? await Journal.query(trx).where('meta,nlmta', journal_nlmta)
      : undefined
    if (journalSearch === undefined || journalSearch.length !== 1) {
      manuscript.meta.unmatchedJournal = journal
    } else {
      manuscript.journalId = journalSearch[0].id
    }
    const existing = pprid
      ? await ManuscriptManager.findByArticleId(
          pprid,
          submitter.userId,
          true,
          trx,
        )
      : { manuscript: null }
    const organizationId = await OrganizationManager.getOrganizationID(
      preprint,
      trx,
    )
    const created = existing.manuscript
      ? { id: null }
      : await ManuscriptManager.create(
          manuscript,
          submitter.userId,
          organizationId,
          trx,
        )
    const manuscriptId = created.id || existing.manuscript.id

    let reviewerNoteId = null

    if (
      existing.manuscript &&
      existing.manuscript.id &&
      parseInt(version, 10) <= parseInt(existing.manuscript.version, 10)
    ) {
      logger.error(
        `Manuscript ${manuscriptId} version ${version} already exists`,
      )
      throw new Error(
        `Error in package ${packageName}. Manuscript version has already been imported`,
      )
    }

    if (permissions) {
      const fileURI = `${tmpPath}/metadata.xml`
      await fs.writeFileSync(fileURI, permissions)
      files.push({
        fileURI,
        filename: 'metadata.xml',
        type: 'metadata',
      })
    }

    if (created.id && reviewers) {
      logger.info(`Adding reviewer for ${manuscriptId}`)
      let xmlReviewer
      if (reviewers.length === 1) {
        // if just one contact, keep this one
        ;[xmlReviewer] = reviewers
      } else {
        const corrpis = reviewers.filter(reviewer => reviewer.corrpi)
        if (corrpis && corrpis.length > 0) {
          ;[xmlReviewer] = corrpis // keep the first corrpi
        } else {
          // if there are not corrpis
          ;[xmlReviewer] = reviewers // keep the first contact
        }
      }
      const reviewer = {
        email: xmlReviewer.email,
        name: {
          givenNames: xmlReviewer.givenNames,
          surname: xmlReviewer.surname,
        },
      }

      if (!validateEmail(xmlReviewer.email)) {
        const error = new Error(
          `Reviewer's email is not valid: ${xmlReviewer.email} in package ${packageName}`,
        )
        error.submitter = submitter
        error.filePath = packagePath
        error.tmpPath = tmpPath
        throw error
      }

      const identities = await Identity.query(trx).where(
        'email',
        xmlReviewer.email,
      )
      if (identities && identities.length > 0) {
        // identity exists
        reviewer.id = identities[0].userId
      }
      const reviewerNote = await NoteManager.create(
        {
          notesType: 'selectedReviewer',
          content: JSON.stringify(reviewer),
          manuscriptId,
          manuscriptVersion: version,
        },
        submitter.userId,
        trx,
      )
      reviewerNoteId = reviewerNote.meta.notes[0].id
    } else if (created.id) {
      const error = new Error(
        `Error in package ${packageName}. There are no reviewers in the XML.`,
      )
      error.submitter = submitter
      error.filePath = packagePath
      error.tmpPath = tmpPath
      throw error
    }

    if (versionList) {
      logger.info(`Adding version information for ${manuscriptId}`)
      let notDupes = []
      let reviewerUserId = null
      const existingNotes = existing.manuscript && existing.manuscript.notes
      const existingNote =
        existingNotes &&
        existingNotes.find(n => n.notesType === 'notDuplicates')
      const oldVersions =
        existingNotes && existingNotes.find(n => n.notesType === 'versionList')
      if (existingNote) {
        notDupes = JSON.parse(existingNote.content)
        await NoteManager.delete(existingNote.id, submitter.userId, trx)
      }
      if (oldVersions) {
        await NoteManager.delete(oldVersions.id, submitter.userId, trx)
      }
      await versionList.reduce(async (promise, verId) => {
        await promise
        const verSearch = await ManuscriptManager.findByArticleId(
          verId,
          submitter.userId,
          true,
          trx,
        )
        const otherVer = verSearch.manuscript
        if (otherVer) {
          const existingTeam = await TeamManager.selectByManIdAndType(
            otherVer.id,
            'reviewer',
            trx,
          )
          if (existingTeam) reviewerUserId = existingTeam.userId
          if (!notDupes.includes(otherVer.id)) {
            notDupes.push(otherVer.id)
          }
          const { notes } = otherVer
          const dupeNote =
            notes && notes.find(n => n.notesType === 'notDuplicates')
          if (dupeNote) {
            const dupeNoteArray = JSON.parse(dupeNote.content)
            if (!dupeNoteArray.includes(manuscriptId)) {
              dupeNoteArray.push(manuscriptId)
            }
            await NoteManager.update(
              {
                id: dupeNote.id,
                content: JSON.stringify(dupeNoteArray),
                manuscriptVersion: otherVer.version,
              },
              submitter.userId,
              trx,
            )
          } else {
            await NoteManager.create(
              {
                notesType: 'notDuplicates',
                content: JSON.stringify([manuscriptId]),
                manuscriptId: otherVer.id,
                manuscriptVersion: otherVer.version,
              },
              submitter.userId,
              trx,
            )
          }
          const verNote =
            notes && notes.find(n => n.notesType === 'versionList')
          if (verNote) {
            const verNoteObj = JSON.parse(verNote.content)
            if (!verNoteObj.emsid.includes(manuscriptId))
              verNoteObj.emsid.push(manuscriptId)
            if (!verNoteObj.pprid.includes(pprid)) verNoteObj.pprid.push(pprid)
            await NoteManager.update(
              {
                id: verNote.id,
                content: JSON.stringify(verNoteObj),
                manuscriptVersion: otherVer.version,
              },
              submitter.userId,
              trx,
            )
          } else {
            await NoteManager.create(
              {
                notesType: 'versionList',
                content: JSON.stringify({
                  pprid: [pprid],
                  emsid: [manuscriptId],
                }),
                manuscriptId: otherVer.id,
                manuscriptVersion: otherVer.version,
              },
              submitter.userId,
              trx,
            )
          }
          return Promise.resolve()
        }
      }, Promise.resolve())
      await NoteManager.create(
        {
          notesType: 'versionList',
          content: JSON.stringify({
            pprid: versionList,
            emsid: notDupes,
          }),
          manuscriptId,
          manuscriptVersion: version,
        },
        submitter.userId,
        trx,
      )
      await NoteManager.create(
        {
          notesType: 'notDuplicates',
          content: JSON.stringify(notDupes),
          manuscriptId,
          manuscriptVersion: version,
        },
        submitter.userId,
        trx,
      )
      if (reviewerUserId && reviewerNoteId) {
        await TeamManager.addNewReviewer(reviewerNoteId, reviewerUserId, trx)
      }
    }

    logger.info(`Uploading files for ${manuscriptId}`)
    let manuscriptFile = {}
    await files.reduce(async (promise, file) => {
      await promise
      const { fileURI: filePath, filename, type, info: label } = file
      const stats = fs.statSync(filePath)
      const { size } = stats
      let extension = path.extname(filePath)
      if (extension === '.tgz') extension = '.gz'
      const mimeType = mime.contentType(extension)
      const uuid = uuidv4()
      // add minio upload delay
      await new Promise(resolve => setTimeout(resolve, 500))
      logger.info(
        `GRIDFS FTP file upload for manuscript ${manuscriptId} version ${version}`,
      )
      await fileUtils.fileUpload(
        manuscriptId,
        version,
        `${uuid}${extension}`,
        filename,
        filePath,
        mimeType,
      )
      const uploadFile = {
        filename,
        label,
        type,
        mimeType,
        size,
        url: `/download/${uuid}${extension}`,
        manuscriptId,
        manuscriptVersion: version,
      }
      if (type === 'manuscript') {
        manuscriptFile = uploadFile
      }
      return FileManager.save(uploadFile, submitter.userId, trx)
    }, Promise.resolve())

    const { url, filename, mimeType } = manuscriptFile
    // case when manuscript is a docx file
    if (
      mimeType ===
      'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
    ) {
      try {
        logger.info(`Starting XSweet conversion of ${filename}`)
        await xsweetConvert(
          url,
          manuscriptId,
          version,
          submitter.userId,
          internalServiceUrl,
          trx,
        )
        logger.info(
          `XSweet conversion of ${filename} for ${manuscriptId} is done`,
        )
      } catch (error) {
        logger.error(`Unable to convert file ${filename}`)
      }
      // case when set to show publisher PDF
    } else if (mimeType === 'application/pdf' && publisherPDF) {
      delete manuscriptFile.id
      manuscriptFile.type = 'pdf4load'
      await FileManager.save(manuscriptFile, submitter.userId, trx)
    }

    const tagging = states.indexOf('tagging')
    const done = states.indexOf('published')
    const status = existing.manuscript && existing.manuscript.status
    const notProcessing =
      status &&
      (states.indexOf(status) < tagging || states.indexOf(status) >= done)
    if (created.id || notProcessing) {
      const submitMan = {
        id: manuscriptId,
        status: 'submitted',
        version,
      }
      logger.info(`FTP submitting manuscript ${manuscriptId}`)
      // trx is committed in .submit()
      await ManuscriptManager.submit(submitMan, submitter.userId, trx)
      logger.info(`Manuscript ${manuscriptId} successfully submitted`)
    } else {
      await trx.commit()
    }
    return manuscriptId
  } catch (err) {
    await trx.rollback()
    throw err
  }
}

const validateEmail = email => {
  const re = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
  return re.test(email)
}

const updateFTP = async (updatedFilepath, submitter) => {
  try {
    const [datedFolder, processedFilename] = await fileUtils.getDirAndFilename(
      updatedFilepath,
    )
    logger.info('Updating FTP server...')

    await ftpUtils.renameBulkFtpPackage(
      submitter ? submitter.meta.ftpUserName : null,
      datedFolder,
      processedFilename,
    )
  } catch (err) {
    logger.error('Error with updating FTP')
  }
}
