const Client = require('ftp')
const logger = require('@pubsweet/logger')
const { execSync } = require('child_process')
const config = require('config')
const fs = require('fs')
const ftpUser = require('../utils/ftpUser')
const { errorDevEmail } = require('../email')
const { checkJobStatus } = require('../job-runner')
const { FtpAccountManager } = require('../xpub-model')

if (!process.env.ENABLE_CRONJOB_FROMFTPTAGGERIMPORT) {
  logger.info(
    'ENABLE_CRONJOB_FROMFTP-TAGGERIMPORT not defined. fromFtp-TaggerImport cronjob exits.',
  )
  process.exit(0)
}

const BULK_UPLOAD_FILE_EXT = new RegExp(/\S+\.tar\.gz$/i)
const ftp = new Client()

;(async () => {
  await checkJobStatus(
    'from-ftp-taggerimport',
    async () => {
      const ftpTagger = await FtpAccountManager.findByFtpUsername(
        'beta_plus_tagger',
      )

      // create local directories
      const parentRootPathLocal = `${process.env.HOME}/${config.get(
        'ftp_directory',
      )}/${ftpTagger.username}`
      const rootPathLocal = `${parentRootPathLocal}/Done`
      const ignoreNewFolder = `${parentRootPathLocal}/New`
      const errorFolder = `${parentRootPathLocal}/Error`

      const cmd = `mkdir -p ${rootPathLocal} ${ignoreNewFolder} ${errorFolder}`
      execSync(cmd)

      logger.info('Tagger Import: downloading files from FTP')
      const beforeUpdate = Date.now()
      try {
        const val = await checkNewFtpPackage(ftpTagger.username, rootPathLocal)
        logger.info(
          `Tagger Import: ${val} file(s) downloaded to local directory ${rootPathLocal} in ${Date.now() -
            beforeUpdate} ms`,
        )
      } catch (err) {
        errorDevEmail('Tagger Import Cron Job', err)
        logger.error(err)
        close(ftp)
        throw new Error(err)
      }
      close(ftp)
    },
    // FTP healthcheck
    true,
  )
  process.exit(0)
})()

function close(ftp) {
  ftp.end()
  ftp = null
  logger.info('Ftp connection terminated')
}

process
  .on('uncaughtException', (err, origin) => {
    logger.error('Uncaught Exception thrown: ', err)
    logger.error('Exception thrown at: ', origin)
    close(ftp)
    process.exit(1)
  })
  .on('unhandledRejection', (reason, promise) => {
    logger.error('Unhandled Rejection: ', reason)
    logger.error('Rejected at promise: ', promise)
    close(ftp)
    process.exit(1)
  })

async function checkNewFtpPackage(ftpUsername, rootPathLocal) {
  const parentRootPathFTP = `${
    config.get('taggers-upload-ftp')['receive-folder']
  }`
  const rootPathFTP = `${parentRootPathFTP}/Done`
  const ignoreNewFolder = `${parentRootPathFTP}/New`
  const errorFolder = `${parentRootPathFTP}/Error`
  const pathArr = [rootPathFTP, ignoreNewFolder, errorFolder]

  const { host } = config.get('taggers-upload-ftp')
  const { username: user, password } = await ftpUser.getFTPAccount(ftpUsername)
  pathArr.forEach(dir => {
    ftp.mkdir(dir, true, err => {
      if (err) {
        logger.error(err)
      }
    })
  })

  return new Promise((resolve, reject) => {
    ftp.on('ready', () => {
      ftp.list(rootPathFTP, false, async (err, list) => {
        if (err) {
          close(ftp)
          reject(err)
        }
        // get array of tar.gz files only
        const targzFiles = list.filter(
          file => BULK_UPLOAD_FILE_EXT.test(file.name) && file.type === '-',
        )
        if (!targzFiles.length) {
          resolve(0) // No files to download to local FTP space
        }
        downloadFtpFiles(targzFiles, ftp, rootPathFTP, rootPathLocal)
          .then(() => resolve(targzFiles.length))
          .catch(err => {
            reject(err.message)
          })
      })
    })
    ftp.on('error', () =>
      reject(new Error('Unable to connect to Bulk Upload FTP')),
    )
    const keepalive = 10000
    const connTimeout = 60000
    const pasvTimeout = 60000
    ftp.connect({ host, user, password, keepalive, connTimeout, pasvTimeout })
  })
}

function downloadFtpFiles(targzFiles, ftp, rootPathFTP, rootPathLocal) {
  return Promise.all(
    targzFiles.map(
      file =>
        new Promise((resolve, reject) => {
          const remoteFilePath = `${rootPathFTP}/${file.name}`
          ftp.get(remoteFilePath, (err, stream) => {
            if (err) {
              logger.error(`${remoteFilePath}: ${err.message}`)
              reject(err.message)
            } else if (!stream) {
              logger.error(`${remoteFilePath}: No FTP stream`)
              reject(err.message)
            } else {
              const path = `${rootPathLocal}/${file.name}`
              const writeStream = stream.pipe(fs.createWriteStream(path))
              writeStream.on('finish', () => {
                logger.info(
                  `Tagger Import: FTP file has been successfully downloaded to local directory ${path}`,
                )
                resolve()
              })
            }
          })
        }),
    ),
  )
}
