const path = require('path')
const logger = require('@pubsweet/logger')
const tar = require('../utils/unTar.js')
const files = require('../utils/files.js')
const ftpUtils = require('../utils/ebi-ftp.js')
const uuidv4 = require('uuid/v4')
const config = require('config')
const {
  FileManager,
  ManuscriptManager,
  IdentityManager,
} = require('../xpub-model/')
const { transaction } = require('objection')
const BaseModel = require('@pubsweet/base-model')
const { pushXML } = require('../xml-validation')

const knex = BaseModel.knex()
const ftpTagger = config.get('ftp_tagger')
const { processedTaggerEmail, bulkUploaderEmail } = require('../email')

const parentRootPath = `${process.env.HOME}/${config.get('ftp_directory')}/${
  ftpTagger.username
}`

const errorFolder = `${parentRootPath}/Error`

module.exports.runProcess = async filePath => {
  logger.info(`Processing tagged XML package: ${filePath}`)
  const fileName = path.basename(filePath)
  const trx = await transaction.start(knex)
  const fileTypes = ['PMC', 'IMGsmall', 'IMGview', 'IMGprint', 'supplement_tag']
  let suffixedFilePath
  try {
    logger.info(`${fileName}: Extracting package`)
    const user = await IdentityManager.findByFtpUsername(
      ftpTagger.username,
      trx,
    )
    const { userId } = user
    const tmpPath = await tar.createTempDir()
    const extractedFilePath = await tar.untar(filePath, tmpPath)
    suffixedFilePath = await files.renameFileSuffix(filePath)
    const manifestFilename = await files.getManifestFilename(extractedFilePath)
    const [manId, filesData] = await files.getManifestFileData(
      extractedFilePath,
      manifestFilename,
    )
    logger.info(`${fileName}: Getting manuscript and file info`)
    const man = await ManuscriptManager.findById(manId, userId, trx, false)
    const { manuscripts } = await ManuscriptManager.getManuscriptVersions(
      [manId],
      trx,
    )
    if (!manuscripts[0]) {
      throw new Error(`Submission ${manId} is no longer available for tagging`)
    }
    const { version } = manuscripts[0]
    const filesArr = await files.checkFiles(filesData, version, tmpPath, userId)

    // delete current existing 'tagging' files
    logger.info(`${fileName}: Updating files in minio and the database`)
    await fileTypes.reduce(async (promise, type) => {
      await promise
      return FileManager.deleteByManIdAndType(manId, type, userId, trx)
    }, Promise.resolve())

    let xmlFile = null
    // upload to minio
    await filesArr.reduce(async (promise, file) => {
      await promise
      const uuid = uuidv4()
      file.minioFileName = `${uuid}${file.extension}`
      await files.fileUpload(
        manId,
        version,
        file.minioFileName,
        file.filename,
        file.url,
        file.mimeType,
      )
      // create rows in 'file' table
      file.url = `/download/${file.minioFileName}`
      delete file.extension
      delete file.minioFileName
      if (file.type === 'PMC') {
        xmlFile = file.url
      }
      return FileManager.save(file, userId, trx)
    }, Promise.resolve())
    // generate HTML and nxml files from xml
    await trx.commit()

    logger.info(`${fileName}: Transforming tagger XML`)

    if (xmlFile) {
      await pushXML(xmlFile, manId, userId)
    } else {
      throw new Error(`${filePath}: No XML file!`)
    }
    logger.info(`${fileName}: Sending success email and tidying.`)
    // send email to taggers
    processedTaggerEmail(ftpTagger.email, manId, man.meta.title, fileName)
    // move file to dated folder
    const datedFilePath = await files.renameFile(suffixedFilePath)

    await updateFTP(datedFilePath)
    // clean up tmp folder
    await files.tidyUp(tmpPath)
    return true
  } catch (err) {
    if (trx) await trx.rollback()
    logger.error('Error', err.message)
    bulkUploaderEmail(ftpTagger.email, err.message, fileName)
    const datedFilePath = await files.moveErroneousFile(
      `${filePath}.start`,
      errorFolder,
    )
    // update FTP - rename files accordingly
    await updateFTP(datedFilePath)
    return true
  }
}

const updateFTP = async datedFilePath => {
  try {
    const [datedFolder, processedFilename] = await files.getDirAndFilename(
      datedFilePath,
    )
    logger.info(`${processedFilename}: Updating FTP server.`)
    let folderName = 'Done'
    if (!datedFolder && datedFilePath.split('/').slice(-2)[0] === 'Error')
      folderName = 'Error'

    await ftpUtils.renameTaggerFtpPackage(
      ftpTagger.username,
      folderName,
      datedFolder,
      processedFilename,
    )
  } catch (err) {
    logger.error(`${datedFilePath}: Error with updating FTP`)
  }
}
