const logger = require('@pubsweet/logger')
const superagent = require('superagent')
const config = require('config')
const { parseExpression } = require('cron-parser')
const { JobManager } = require('../xpub-model/')

const { DATA_CENTER } = process.env
const { internalServiceUrl } = config.get('pubsweet-server')

const checkJobStatus = async (name, logic, ftp) => {
  // For local running, skip healthcheck if data center not defined
  if (DATA_CENTER) {
    // Perform healthcheck
    logger.info(`${name}: Running healthcheck`)
    const url = `${internalServiceUrl}/healthcheck?datacenterName=${DATA_CENTER}&ftpExcluded=${
      ftp ? 0 : 1
    }`
    try {
      logger.info('Healthcheck url: ', url)
      await superagent.get(url)
    } catch (error) {
      const { response, message } = error
      const { body, status } = response
      logger.error('Healthcheck failed. Exiting.')
      logger.info(`Status code: ${status}`)
      logger.info('Message: ', message)
      logger.info('Response body: ', body)
      return false
    }
  }
  // Get job status from the database
  logger.info(`Checking ${name} status`)
  let job = {}
  try {
    job = await JobManager.find(name)
  } catch {
    logger.error('Job not found in database. Exiting.')
    return false
  }
  const { running, lastPass, frequency } = job
  if (running) {
    logger.info(`${name} is already running. Exiting.`)
    return false
  }
  // Check if next pass exists between lastPass and current time
  const testRange = {
    currentDate: new Date(lastPass),
    endDate: new Date(),
    iterator: true,
  }
  const interval = await parseExpression(frequency, testRange)
  try {
    await interval.next()
    logger.info(`${name} scheduled. Running:`)
  } catch (ignore) {
    logger.info(`${name} not yet scheduled to run. Exiting.`)
    return false
  }
  // Mark job as running
  await JobManager.update({ name, running: true })
  // Run job logic
  let status = 'fail'
  try {
    await logic()
    status = 'pass'
  } catch {
    status = 'fail'
  }
  // Update status when job completes (or fails)
  return JobManager.update({
    name,
    running: false,
    lastStatus: status,
    lastPass: status === 'pass' ? new Date().toISOString() : lastPass,
  })
}

module.exports = { checkJobStatus }
