const config = require('config')
const logger = require('@pubsweet/logger')
const Organization = require('../xpub-model/entities/organization')
const Manuscript = require('../xpub-model/entities/manuscript/data-access')
const { parsed, sendMail } = require('../email')
const { createManifest } = require('../ebi-integration/toEbi')
const { xmlChange } = require('../xml-validation')
const { checkJobStatus } = require('../job-runner')
const getUser = require('../utils/user.js')

const { url, system } = config['epmc-email']

function followUpTemplate(id, salutation, title, link, existing, fortnight) {
  return `
  <p style="margin:-32px 0 -12px; text-align: right; font-size: 12px;">${id}</p>
  <p>Dear ${salutation}</p>
  <p>We recently emailed asking you to check and approve the formatting of your preprint, <b>${title}</b>. Because your preprint has a Creative Commons license, the full text will automatically be displayed on the Europe PMC website for people to read on ${fortnight}.</p>
  <p>If you would rather check and approve the web format, it can be made available for reading on the Europe PMC website within 24 hours. To check the preprint before it goes live, please log into ${
    existing ? ' your' : ' or create a'
  } Europe PMC plus account by clicking the button below.</p>
  <p style="text-align:center"><a style="display: inline-block; font-size: 20px; padding: 14px 18px 12px; background-color:#20699C; color: #fff; font-weight: 300; border-radius: 1px; text-decoration: none" href="${link}">Review my preprint display</a></p>
  <p>If you have questions, please see the <a href="https://plus.europepmc.org/user-guide/preprintfaqs" style="color:#20699C">Preprint FAQs</a>.</p>
  <p>Kind regards,</p>
  <p>The Europe PMC Helpdesk</p>
`
}

async function followUpEmail(reviewer, manInfo, token) {
  const { id, title } = manInfo
  const { title: t, givenNames: g, surname: s } = reviewer.name
  const salutation = `${t ? `${t} ` : ''}${g} ${s}`
  let existing = true
  let link = `${url}submission/${id}/review`
  if (token && !reviewer.id) {
    existing = false
    link = `${url}dashboard?accept=${token}`
  }
  const fortnightDate = new Date(Date.now() + 12096e5)
  const fortnight = `${fortnightDate.getUTCDate()} ${fortnightDate.toLocaleString(
    'en-GB',
    { month: 'long' },
  )}`
  const html = followUpTemplate(
    id,
    salutation,
    parsed(title),
    link,
    existing,
    fortnight,
  )
  if (html) {
    const subject = `Display of your preprint in the Europe PMC archive.`
    return sendMail(reviewer.email, subject, html, null, null, system)
  }
}

async function sendFollowUp(manuscript) {
  const { id, teams, notes } = manuscript
  const title = manuscript['meta,title']
  const revTeam = teams.find(team => team.roleName === 'reviewer')
  const revUser = revTeam && revTeam.users && revTeam.users[0]
  try {
    let reviewer = null
    let token = null
    if (revUser && revUser.id) {
      const { title, givenNames, surname, identities } = revUser
      const { email, userId } = identities[0]
      reviewer = { id: userId, name: { title, givenNames, surname }, email }
    } else {
      const revNote = notes.find(n => n.notesType === 'selectedReviewer')
      reviewer = revNote && JSON.parse(revNote.content)
      token = revNote && revNote.id
    }
    logger.info(`Sending email and updating ${id}`)
    if (reviewer.email) {
      await followUpEmail(reviewer, { id, title }, token)
    } else {
      logger.error(`No reviewer email for ${id}`)
    }
    return Manuscript.query()
      .patch({ ebiState: 'Open access preprint clock reset' })
      .where('id', id)
  } catch (err) {
    logger.error(`Unable to update ${id}: ${err.message}`)
    return Promise.resolve()
  }
}

async function preprintsEmailCheck() {
  logger.info(`Running script to reset clocks for CC preprints`)
  const orgId = await Organization.getOrganizationID(true)
  const manuscripts = await Manuscript.query()
    .joinRelation('notes')
    .where('notes.notesType', 'openAccess')
    .whereNull('notes.deleted')
    .where('manuscript.status', 'xml-review')
    .where('manuscript.organizationId', '=', orgId)
    .where('manuscript.ebiState', '=', 'Send update email')
    .whereNull('manuscript.deleted')
    .eager('[notes, teams.users.identities]')
    .modifyEager('notes', e => {
      e.whereNull('deleted')
    })
    .modifyEager('teams', e => {
      e.whereNull('deleted')
    })
  return manuscripts.reduce(async (promise, m) => {
    await promise
    return sendFollowUp(m)
  }, Promise.resolve())
}

async function repoResendCheck() {
  logger.info(
    `Running script to silently resend manuscripts to the EBI repository`,
  )
  const adminUser = await getUser.getAdminUser()
  const manuscripts = await Manuscript.query()
    .where('manuscript.ebiState', '=', 'Repo resend')
    .whereNull('manuscript.deleted')
  return manuscripts.reduce(async (promise, m) => {
    await promise
    return createManifest(m.id, adminUser.id)
  }, Promise.resolve())
}

async function xmlChangeCheck() {
  logger.info(`Running script to edit XMLs en masse`)
  const manuscripts = await Manuscript.query()
    .where('manuscript.ebiState', '=', 'Mass edit XML')
    .whereNull('manuscript.deleted')
  return manuscripts.reduce(async (promise, m) => {
    await promise
    return xmlChange(m.id, m.version)
  }, Promise.resolve())
}

;(async () => {
  await checkJobStatus('special-db-checks', async () => {
    const updateTime = Date.now()
    logger.info('Checking for special update needs')
    await preprintsEmailCheck()
    await repoResendCheck()
    await xmlChangeCheck()
    logger.info(`Checks were finished in ${Date.now() - updateTime} ms`)
  })
  Manuscript.knex().destroy()
  process.exit()
})()
