const path = require('path')
const libxml = require('libxmljs')
const xslt4node = require('xslt4node')
const fs = require('fs')
const uuidv4 = require('uuid/v4')
const logger = require('@pubsweet/logger')
const config = require('config')
const { transaction } = require('objection')
const BaseModel = require('@pubsweet/base-model')
const tar = require('../utils/unTar.js')
const fileUtils = require('../utils/files.js')
const getUser = require('../utils/user.js')
const { createManifest } = require('../ebi-integration/toEbi')
const Manuscript = require('../xpub-model/entities/manuscript')
const File = require('../xpub-model/entities/file')
const Note = require('../xpub-model/entities/note')

const { baseUrl } = config.get('pubsweet-server')
const xlink = 'http://www.w3.org/1999/xlink'
const ali = 'http://www.niso.org/schemas/ali/1.0/'
const knex = BaseModel.knex()

xslt4node.addLibrary('./saxon9he.jar')
xslt4node.addOptions('-Xmx1g')

const addCitation = async function addCitation(manId) {
  const adminUser = await getUser.getAdminUser()
  const userId = adminUser.id
  const trx = await transaction.start(knex)
  try {
    const manuscript = await Manuscript.findById(manId, userId, trx)
    const [xml] = await File.findByManIdAndType(manId, 'PMC', trx)
    if (xml) {
      const [pf] = await File.findByManIdAndType(manId, 'metadata', trx)
      const permissions = pf && baseUrl + pf.url
      logger.info(`Permission XML transformed ${permissions}`)
      const { meta, version, journal } = manuscript
      const { articleIds, volume, issue, notes, publicationDates } = meta
      const { journalTitle } = journal
      const { issn, nlmta } = journal.meta
      const { fpage, lpage, elocationId } = meta.location || {}
      const xmlString = await fileUtils.fetchFile(baseUrl + xml.url)
      const xmlIsWellformed = libxml.parseXml(xmlString)
      logger.info('XML to transform read from DB')
      const { externalId, systemId } = await xmlIsWellformed.getDtd()
      const license = notes && notes.find(n => n.notesType === 'openAccess')
      const licenseLink = license && license.content
      const openAccess =
        licenseLink && licenseLink.search(/\/(by-nd|by-nc-nd)\//i) < 0
      const addLicense = await Note.getLatestValue(manId, 'addLicense', trx)
      const params = {
        externalId,
        systemId,
        journalTitle: journalTitle || '',
        nlmta: nlmta || '',
        volume: volume || '',
        issue: issue || '',
        fpage: fpage || '',
        lpage: lpage || '',
        elocationId: elocationId || '',
        permissions: permissions || '',
        addLicense: (!openAccess && addLicense) || '',
        version: (version > 0 && Math.round(version)) || '',
      }
      issn.forEach(n => {
        const t = n.type.toLowerCase().charAt(0)
        params[`${t}issn`] = n.id
      })
      articleIds.forEach(aid => {
        if (aid.pubIdType !== 'pre-pmc') {
          params[aid.pubIdType] = aid.id
        }
      })
      publicationDates.forEach(pd => {
        params[pd.type] = pd.jatsDate
          ? Object.keys(pd.jatsDate)
              .filter(el => pd.jatsDate[el])
              .sort()
              .map(el => `${el}=${pd.jatsDate[el]}|`)
              .join('')
          : ''
      })
      logger.info('XML generated. Trying to transform')
      const newXml = await transformXML(
        xmlString,
        path.resolve(__dirname, 'xsl/addcitation.xsl'),
        params,
      )
      logger.info('XML transformation done')
      const tmpPath = await tar.createTempDir()
      const tmpFilePath = `${tmpPath}/${manId}.xml`
      fs.writeFileSync(tmpFilePath, newXml)
      const uuid = uuidv4()
      const mimeType = 'application/xml'
      const newPath = `/download/${uuid}.xml`
      logger.info(
        `GRIDFS Uploading file XML transformed stored in GrdiFS for manusctipt ${manId} with version ${version}`,
      )
      const fileSize = fs.statSync(tmpFilePath).size
      await fileUtils.fileUpload(
        manId,
        version,
        `${uuid}.xml`,
        `${manId}.xml`,
        tmpFilePath,
        mimeType,
      )

      const fileInfo = {
        id: xml.id,
        url: newPath,
        filename: `${manId}.xml`,
        type: 'PMC',
        label: '',
        size: fileSize,
        mimeType,
        manuscriptId: manId,
      }
      await File.save(fileInfo, userId, trx)
      logger.info('XML uploaded')
      await trx.commit()
      await pushXML(newPath, manId, userId)
      logger.info('XML transformation process to add to citation is done')
    } else {
      throw new Error('No XML file found')
    }
  } catch (err) {
    if (trx) {
      await trx.rollback()
    }
    await Manuscript.update(
      {
        id: manId,
        formState: `Error adding citation to XML: ${err.message}`,
        status: 'repo-triage',
        pdfDepositState: null,
      },
      userId,
    )
    logger.error(err)
  }
}

const pushXML = async function pushXML(
  fileUrl,
  manuscriptId,
  userId,
  keepStatus,
) {
  const adminUser = await getUser.getAdminUser()
  await Manuscript.update(
    { id: manuscriptId, formState: null, pdfDepositState: 'CREATING_NXML' },
    userId,
  )
  const trx = await transaction.start(knex)
  const man = await Manuscript.findById(manuscriptId, adminUser.id, trx)
  const errorStatus =
    (keepStatus && man.status) ||
    (['tagging'].includes(man.status) && 'file-error') ||
    (['repo-ready', 'repo-processing'].includes(man.status) && 'repo-triage') ||
    man.status
  try {
    const xml = await fileUtils.fetchFile(baseUrl + fileUrl)
    const xmlIsWellformed = libxml.parseXml(xml)
    const { systemId } = await xmlIsWellformed.getDtd()
    const [sysId] = systemId.split('.')

    const xsdBlue = await fileUtils.readData(
      path.resolve(__dirname, `xsl/xsd/publishing/${sysId}.xsd`),
    )

    const baseUrlB = path.resolve(__dirname, 'xsl/xsd/publishing/')
    const xsdBlueDoc = libxml.parseXml(xsdBlue, {
      baseUrl: `${baseUrlB}/`,
    })

    const xmlIsValid = xmlIsWellformed.validate(xsdBlueDoc)

    const errors = xmlIsWellformed.validationErrors

    if (!xmlIsValid) {
      let errString = 'Invalid XML: \n'
      errors.forEach((err, i) => {
        errString += `${err}\nLine: ${err.line}.`
        if (err.str1) {
          errString += ` ID: ${err.str1}.`
        }
        if (i !== errors.length - 1) {
          errString += `\n\n`
        }
      })
      throw new Error(errString)
    }

    // Check XML against the stylechecker
    const checked = await transformXML(
      xml,
      path.resolve(__dirname, 'xsl/stylechecker/nlm-stylechecker.xsl'),
    )

    const result = libxml.parseXml(checked)

    const styleErrors = result.find('//error')

    if (styleErrors.length > 0) {
      let styleErrString = `Style Errors: <br/><br/>`
      styleErrors.forEach((err, i) => {
        styleErrString += err
          .text()
          .replace(/</g, '&lt;')
          .replace(/>/g, '&gt;')
        if (i !== styleErrors.length - 1) {
          styleErrString += `<br/><br/>`
        }
      })
      throw new Error(styleErrString)
    }

    const nxml = await transformXML(
      xml,
      path.resolve(__dirname, 'xsl/pnihms2pmc3.xsl'),
    )

    const nxmlIsWellformed = libxml.parseXml(nxml)
    const { systemId: nxmlSystemId } = await nxmlIsWellformed.getDtd()
    const [nxmlId] = nxmlSystemId.split('.')
    const xsdGreen = await fileUtils.readData(
      path.resolve(__dirname, `xsl/xsd/archiving/${nxmlId}.xsd`),
    )
    const baseUrlG = path.resolve(__dirname, 'xsl/xsd/archiving/')
    const xsdGreenDoc = libxml.parseXml(xsdGreen, {
      baseUrl: `${baseUrlG}/`,
    })
    const nxmlIsValid = nxmlIsWellformed.validate(xsdGreenDoc)
    const nxmlErrors = nxmlIsWellformed.validationErrors

    if (!nxmlIsValid) {
      let errString = 'Invalid NXML: \n\n'
      nxmlErrors.forEach((err, i) => {
        errString += `${err}\nLine: ${err.line}.`
        if (err.str1) {
          errString += ` ID: ${err.str1}.`
        }
        if (i !== nxmlErrors.length - 1) {
          errString += `\n\n`
        }
      })
      throw new Error(errString)
    }

    const manuscriptFiles = await File.findByManuscriptId(manuscriptId, trx)
    const filelist = getFilelist(manuscriptFiles)

    const tmpPath = await tar.createTempDir()

    // now we will transform the nXML into HTML:

    let xsl = ''

    try {
      xsl = await fileUtils.fetchFile(
        'https://gitlab.ebi.ac.uk/literature-services/public-projects/full-text-xsl/-/raw/master/jats2html.xsl',
      )
    } catch (err) {
      throw new Error('JATS to HTML conversion XSL file not accessible')
    }
    try {
      const html = await transformXML(
        nxml,
        null,
        { filelist, msspreview: 'true()' },
        xsl,
      )
      // save converted.html to tmp folder
      fs.writeFileSync(`${tmpPath}/${manuscriptId}.html`, html)
    } catch (err) {
      throw new Error(
        'Unable to convert NXML to HTML - problem with NXML or XSL\n',
        err.message,
      )
    }

    // save converted.nxml to tmp folder
    fs.writeFileSync(`${tmpPath}/${manuscriptId}.nxml`, nxml)

    const filesArr = [
      getFileInfo(`${tmpPath}/${manuscriptId}.html`, adminUser.id),
      getFileInfo(`${tmpPath}/${manuscriptId}.nxml`, adminUser.id),
    ]

    await filesArr.reduce(async (promise, file) => {
      await promise
      const uuid = uuidv4()
      logger.info(
        `GRIDFS Uploading file ARRAY ${file.filename} for masnuscript ${manuscriptId} manuscript version ${man.version}`,
      )
      await fileUtils.fileUpload(
        manuscriptId,
        man.version,
        `${uuid}${file.extension}`,
        file.filename,
        file.url,
        file.mimeType,
      )

      file.url = `/download/${uuid}${file.extension}`
      file.manuscriptId = manuscriptId
      file.manuscriptVersion = man.version
      delete file.extension
      const [fileExists] = await File.findByManIdAndType(
        manuscriptId,
        file.type,
        trx,
      )
      if (fileExists) {
        file.id = fileExists.id
      }
      return File.save(file, adminUser.id, trx)
    }, Promise.resolve())

    await fileUtils.tidyUp(tmpPath)
    logger.debug(
      'Uploading of nxml and html files to GridFS and the database has been completed.',
    )

    const xmlDoc = libxml.parseXml(xml)

    // Get article-type
    const articleType = xmlDoc.get('/article/@article-type').value()

    // Check for open access
    const [
      license,
    ] = xmlDoc.find(
      '//descendant-or-self::license[contains(@xlink:href, "creativecommons.org") or contains(ali:license_ref, "creativecommons.org")]/descendant-or-self::*[self::ali:license_ref or attribute::xlink:href]',
      { xlink, ali },
    )
    const { notes } = man.meta
    const oAnote = notes && notes.find(n => n.notesType === 'openAccess')
    const planS = notes && notes.find(n => n.notesType === 'planS')
    let licenseLink = null
    let isOpen = false
    if (license) {
      licenseLink = license.get('@xlink:href', { xlink })
        ? license.get('@xlink:href', { xlink }).value()
        : license.text()
      isOpen =
        licenseLink.search(
          /\/(publicdomain|by|by-nc|by-nd|by-sa|by-nc-sa|by-nc-nd)\//i,
        ) >= 0
      if (isOpen) {
        if (!oAnote) {
          await Note.create(
            {
              notesType: 'openAccess',
              content: licenseLink,
              manuscriptId,
              manuscriptVersion: man.version,
            },
            adminUser.id,
            trx,
          )
        } else if (oAnote.content !== licenseLink) {
          await Note.update(
            {
              id: oAnote.id,
              content: licenseLink,
              manuscriptVersion: man.version,
            },
            adminUser.id,
            trx,
          )
        }
      } else if (oAnote) {
        await Note.delete(oAnote.id, adminUser.id, trx)
      }
    } else if (oAnote) {
      await Note.delete(oAnote.id, adminUser.id, trx)
    }

    // ***********************
    // Commit database changes
    // ***********************

    await trx.commit()

    // *****************
    // File error checks
    // *****************

    // Check for preprint withdrawal/removal

    if (articleType === 'preprint') {
      // Find third paragraph
      const ptest = xmlDoc.find('//p')
      if (ptest.length <= 2) {
        throw new Error(
          `🔎 May be preprint withdrawal or removal notice. Please check, and if so change article type:\n\n\t• to 'preprint-withdrawal' if previous versions are accessible on the server\n\t• to 'preprint-removal' otherwise\n\nAfter XML processing with the correct article type, withdrawal and removal notices will be released immediately.`,
        )
      }
    }

    // Check for errors in licensing

    if (isOpen) {
      if (planS) {
        if (planS.content) {
          if (!licenseLink.includes(`/${planS.content}/`)) {
            throw new Error(
              `License in XML does not match Plan S exception (CC ${planS.content
                .toUpperCase()
                .replace(/\//g, ' ')})`,
            )
          }
        } else if (!licenseLink.includes('/by/')) {
          throw new Error(
            'Submission with Plan S funding does not have CC BY license.',
          )
        }
      }
    }
    if (planS && !license) {
      throw new Error('Submission with Plan S funding has no license.')
    }
    if (planS && !licenseLink) {
      throw new Error('Submission with Plan S funding has no license URL.')
    }

    // Check for missing referenced files in xml

    // check for @xlink:href attribute
    const nodes = xmlDoc.find(
      '//*[self::media or self::graphic][@xlink:href]',
      { xlink },
    )

    const fileNotFound = []
    let fileFound = false

    nodes.forEach(node => {
      const nodeTag = node.name()

      const filename = node.get('@xlink:href', { xlink }).value()

      if (nodeTag === 'media') {
        // check entire filename including extension
        fileFound = manuscriptFiles.find(el => el.filename === filename)
        if (fileFound === undefined) fileNotFound.push(filename)
      } else if (nodeTag === 'graphic') {
        // check that filename.tif and filename.jpg exist

        const tifFound = manuscriptFiles.find(
          el => el.filename === `${filename}.tif`,
        )

        const jpgFound = manuscriptFiles.find(
          el => el.filename === `${filename}.jpg`,
        )

        if (tifFound === undefined) fileNotFound.push(`${filename}.tif`)
        if (jpgFound === undefined) fileNotFound.push(`${filename}.jpg`)
        fileFound = tifFound && jpgFound
      }
    })

    if (fileNotFound.length > 0) {
      let errString = 'Files referenced in the xml are not in the database: \n'
      fileNotFound.forEach(file => {
        errString += `${file}\n`
      })
      throw new Error(errString)
    }

    // *********************
    // Successful processing
    // *********************

    // check for PDF, and/or whether it's time to send to repo

    const hasPDF = manuscriptFiles.some(f => f.type === 'pdf4load')

    const makePDF = !(hasPDF || man.status === 'repo-ready')

    const obj = {
      id: manuscriptId,
      meta: { articleType },
      pdfDepositState: makePDF ? 'WAITING_FOR_PDF_CONVERSION' : null,
      formState:
        (makePDF && keepStatus && 'Generating PDF') ||
        (articleType.startsWith('preprint-') &&
          'Preprint withdrawal and removal notices will be released immediately.') ||
        null,
    }

    if (man.status === 'tagging' && hasPDF && !keepStatus) {
      obj.status = 'xml-qa'
    }

    const updated = await Manuscript.review(obj, adminUser.id)
    if (
      updated.manuscript.status === 'repo-ready' &&
      man.status !== 'repo-ready'
    ) {
      await addCitation(man.id)
    }
    if (man.status === 'repo-ready') {
      await createManifest(man.id, adminUser.id)
    }
  } catch (err) {
    if (trx) {
      await trx.rollback()
    }
    await Manuscript.update(
      {
        id: manuscriptId,
        formState: err.message,
        status: errorStatus,
        pdfDepositState: null,
      },
      adminUser.id,
    )
    logger.error(err.message)
  }
}

function getFileInfo(filepath, userId) {
  const filename = filepath.substring(filepath.lastIndexOf('/') + 1)
  const fileExt = path.extname(filepath)
  const fileSize = fs.statSync(filepath).size

  const fileInfo = {
    url: filepath,
    filename,
    type:
      (fileExt === '.nxml' && 'PMCfinal') ||
      (fileExt === '.html' && 'tempHTML') ||
      (fileExt === '.xml' && 'PMC'),
    label: '',
    size: fileSize,
    extension: fileExt,
    mimeType:
      (fileExt === '.html' && 'text/html; charset=utf-8') || 'application/xml',
    updatedBy: userId,
  }
  return fileInfo
}

function transformXML(xmlString, xslPath, params, xsl) {
  xmlString = xmlString.replace(/<!DOCTYPE[^>[]*(\\[[^]]*\\])?>/, '')
  const config = {
    xsltPath: xslPath,
    source: xmlString,
    result: String,
    props: {
      indent: 'yes',
    },
  }
  if (!xslPath && xsl) {
    delete config.xsltPath
    config.xslt = xsl
  }
  if (params) {
    config.params = params
  }
  return new Promise((resolve, reject) => {
    xslt4node.transform(config, (err, result) => {
      if (err) {
        reject(err)
      } else {
        resolve(result)
      }
    })
  })
}

function getFilelist(files) {
  const types = ['IMGview', 'supplement', 'supplement_tag']
  const list = files.reduce((list, file) => {
    if (!file.deleted && types.includes(file.type)) {
      if (file.type === 'supplement') {
        file.filename = fileUtils.normalizeFilename(file)
      }
      list.push(file)
    }
    return list
  }, [])
  return list
    .map(
      file =>
        `${file.filename.substring(
          0,
          file.filename.lastIndexOf('.'),
        )}:/api/files/${file.url.split('/').pop()};`,
    )
    .join('')
}

const xmlChange = async function xmlChange(manId, manVersion) {
  const adminUser = await getUser.getAdminUser()
  const userId = adminUser.id
  const trx = await transaction.start(knex)
  try {
    const [xml] = await File.findByManIdAndType(manId, 'PMC', trx)
    if (xml) {
      const xmlString = await fileUtils.fetchFile(baseUrl + xml.url)
      const xmlIsWellformed = libxml.parseXml(xmlString)
      const { externalId, systemId } = await xmlIsWellformed.getDtd()
      const params = { externalId, systemId }
      const newXml = await transformXML(
        xmlString,
        path.resolve(__dirname, 'xsl/changexml.xsl'),
        params,
      )
      logger.info(`${manId} XML changed`)
      const tmpPath = await tar.createTempDir()
      const tmpFilePath = `${tmpPath}/${manId}.xml`
      fs.writeFileSync(tmpFilePath, newXml)
      const uuid = uuidv4()
      const mimeType = 'application/xml'
      const newPath = `/download/${uuid}.xml`
      logger.info(
        `GRIDFS Uploading file XML ${manId} for manuscipt ${manId} version ${manVersion}`,
      )

      const fileSizeXML = fs.statSync(tmpFilePath).size
      await fileUtils.fileUpload(
        manId,
        manVersion,
        `${uuid}.xml`,
        `${manId}.xml`,
        tmpFilePath,
        mimeType,
      )

      const fileInfo = {
        id: xml.id,
        url: newPath,
        filename: `${manId}.xml`,
        type: 'PMC',
        label: '',
        size: fileSizeXML,
        mimeType,
        manuscriptId: manId,
      }
      await File.save(fileInfo, userId, trx)
      logger.info('Changed XML uploaded')
      await trx.commit()
      await pushXML(newPath, manId, userId)
    } else {
      throw new Error('No XML file found')
    }
  } catch (err) {
    if (trx) {
      await trx.rollback()
    }
    await Manuscript.update(
      {
        id: manId,
        formState: `Error changing XML: ${err.message}`,
        pdfDepositState: null,
      },
      userId,
    )
    logger.error(err)
  }
}

module.exports = { addCitation, xmlChange, pushXML }
