const { Model, raw } = require('objection')
const uuid = require('uuid')
const EpmcBaseModel = require('../epmc-base-model')
const { rowToEntity, buildQuery, runQuery } = require('../util')

class File extends EpmcBaseModel {
  static get tableName() {
    return 'file'
  }

  static get schema() {
    return {
      properties: {
        id: { type: 'uuid' },
        manuscriptId: { type: 'string' },
        manuscriptVersion: { type: 'numeric' },
        filename: { type: 'string' },
        mimeType: { type: 'string' },
        type: { type: 'string' },
        size: { type: 'int' },
        url: { type: 'string' },
        label: { type: ['string', 'null'] },
        deleted: { type: 'timestamp' },
        updated: { type: 'timestamp' },
        updatedBy: { type: 'uuid' },
      },
    }
  }

  static get relationMappings() {
    const Manuscript = require('../manuscript/data-access')
    return {
      manuscript: {
        relation: Model.BelongsToOneRelation,
        modelClass: Manuscript,
        join: {
          from: 'file.manuscriptId',
          to: 'manuscript.id',
        },
      },
    }
  }

  static async selectById(id) {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('file')
        .where({ id })
        .whereNull('deleted'),
    )
    if (!rows.length) {
      throw new Error('File not found')
    }
    return rowToEntity(rows[0])
  }

  static async selectByUrl(url) {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('file')
        .where('url', 'like', `%${url}`),
    )
    if (!rows.length) {
      throw new Error('File not found')
    }
    return rowToEntity(rows[0])
  }

  static async selectByManuscriptId(manId, trx) {
    const rows = await File.query(trx)
      .select('*')
      .where('file.manuscript_version', builder => {
        builder
          .select('version')
          .from('manuscript')
          .where('manuscript.id', manId)
      })
      .where('file.manuscript_id', manId)
      .whereNull('file.deleted')
    return rows.map(rowToEntity)
  }

  static async selectByManIdAndType(manId, type, trx) {
    const rows = await File.query(trx)
      .select('*')
      .where('file.manuscript_version', builder => {
        builder
          .select('version')
          .from('manuscript')
          .where('manuscript.id', manId)
      })
      .where('file.manuscript_id', manId)
      .andWhere('file.type', type)
      .whereNull('file.deleted')
    return rows.map(rowToEntity)
  }

  static async selectAllByManuscriptId(manId, trx) {
    const rows = await File.query(trx)
      .select('*')
      .where('file.manuscript_id', manId)
      .whereNull('file.deleted')
    return rows.map(rowToEntity)
  }

  static async selectAll() {
    const rows = await runQuery(
      buildQuery
        .select()
        .from('file')
        .whereNull('deleted'),
    )
    return rows.map(rowToEntity)
  }

  static async selectMigratedUnpublished() {
    const files = await File.query()
      .select('file.*')
      .join('manuscript', 'manuscript.id', 'file.manuscript_id')
      .where('file.filename', 'like', '%.docx')
      .where('file.type', 'manuscript')
      .where(
        raw('cast(substring(file.manuscript_id from 4) as int) < ?', 90000),
      )
      .where(raw('?? <> ?', ['manuscript.status', 'published']))
      .whereNull('file.deleted')
      .whereNull('manuscript.deleted')
    return files
  }

  static async insert(file, userId, trx) {
    file.updatedBy = userId
    file.id = uuid.v4()
    await File.query(trx).insert(file)
    return file.id
  }

  static update(file, userId, trx) {
    file.updatedBy = userId
    return File.query(trx)
      .update(file)
      .where('id', file.id)
      .whereNull('deleted')
  }

  static delete(id, userId) {
    return File.query().updateAndFetchById(id, {
      deleted: new Date().toISOString(),
      updatedBy: userId,
    })
  }

  static deleteByManIdAndType(manuscript_id, type, userId, trx) {
    return File.query(trx)
      .update({
        deleted: new Date().toISOString(),
        updatedBy: userId,
      })
      .where('file.manuscript_version', builder => {
        builder
          .select('version')
          .from('manuscript')
          .where('manuscript.id', manuscript_id)
      })
      .where({ manuscript_id, type })
      .whereNull('deleted')
  }

  static async deleteByManuscriptId(id, userId, trx) {
    await File.query(trx)
      .update({
        deleted: new Date().toISOString(),
        updatedBy: userId,
      })
      .where('file.manuscript_version', builder => {
        builder
          .select('version')
          .from('manuscript')
          .where('manuscript.id', id)
      })
      .where('file.manuscript_id', id)
      .whereNull('deleted')
  }

  static async deleteById(id, userId, trx) {
    return File.query(trx).updateAndFetchById(id, {
      deleted: new Date().toISOString(),
      updatedBy: userId,
    })
  }
}
module.exports = File
