const { transaction, Model, raw } = require('objection')
const logger = require('@pubsweet/logger')
const config = require('config')
const EpmcBaseModel = require('../epmc-base-model')
const BaseModel = require('@pubsweet/base-model')

const knex = BaseModel.knex()

const PAGE_SIZE = config.pageSize || 50

class Manuscript extends EpmcBaseModel {
  static get tableName() {
    return 'manuscript'
  }

  static get schema() {
    return {
      properties: {
        id: { type: 'string', format: 'regex', pattern: 'EMS\\d+' },
        organizationId: { type: 'uuid' },
        journalId: { type: 'uuid' },
        created: { type: 'timestamp' },
        updated: { type: 'timestamp' },
        deleted: { type: 'timestamp' },
        updatedBy: { type: 'uuid' },
        claimedBy: { type: ['string', 'null'] },
        version: { type: 'numeric' },
        status: { type: ['string', 'null'] },
        formState: { type: ['string', 'null'] },
        decision: { type: ['string', 'null'] },
        pdfDepositId: { type: 'uuid' },
        pdfDepositState: { type: ['string', 'null'] },
        retryAttempt: { type: ['numeric'] },
        ncbiState: { type: ['string', 'null'] },
        ebiState: { type: ['string', 'null'] },
        'meta,title': { type: ['string', 'null'] },
        'meta,articleType': { type: ['string', 'null'] },
        'meta,articleIds': {
          type: ['array', 'null'],
          items: {
            type: 'object',
            properties: {
              pubIdType: { type: 'string' },
              id: { type: 'string' },
            },
          },
        },
        'meta,abstract': { type: ['string', 'null'] },
        'meta,subjects': { type: ['string', 'null'] },
        'meta,publicationDates': {
          type: ['array', 'null'],
          items: {
            type: 'object',
            properties: {
              type: { type: 'string' },
              date: { type: 'string' },
              jatsDate: {
                type: 'object',
                properties: {
                  year: { type: 'string' },
                  month: { type: 'string' },
                  day: { type: 'string' },
                  season: { type: 'string' },
                },
              },
            },
          },
        },
        'meta,notes': {
          type: ['array', 'null'],
          items: {
            type: ['object', 'null'],
            properties: {
              id: { type: 'string' },
              content: { type: 'string' },
              notesType: { type: 'string' },
            },
          },
        },
        'meta,volume': { type: ['string', 'null'] },
        'meta,issue': { type: ['string', 'null'] },
        'meta,location': {
          type: ['object', 'null'],
          properties: {
            fpage: { type: ['string', 'null'] },
            lpage: { type: ['string', 'null'] },
            elocationId: { type: ['string', 'null'] },
          },
        },
        'meta,fundingGroup': {
          type: ['array', 'null'],
          items: {
            type: 'object',
            properties: {
              fundingSource: { type: 'string' },
              awardId: { type: 'string' },
              title: { type: ['string', 'null'] },
              categories: {
                type: ['array', 'null'],
                items: {
                  type: ['string', 'null'],
                },
              },
              pi: {
                type: 'object',
                properties: {
                  surname: { type: 'string' },
                  givenNames: { type: ['string', 'null'] },
                  title: { type: ['string', 'null'] },
                  email: { type: 'string' },
                },
              },
              startDate: { type: ['string', 'null'], format: 'date-time' },
              dateChecked: { type: ['boolean', 'null'] },
            },
          },
        },
        'meta,releaseDelay': { type: ['string', 'null'] },
        'meta,unmatchedJournal': { type: ['string', 'null'] },
        'meta,citerefUrl': { type: ['string', 'null'] },
        'meta,fulltextUrl': { type: ['string', 'null'] },
        teams: {
          type: 'array',
          items: {
            type: 'object',
            properties: {
              role: { type: 'string' },
              teamMembers: { type: 'array' },
            },
          },
        },
        files: {
          type: 'array',
          items: {
            type: 'object',
            properties: {
              id: { type: 'string' },
              type: { type: 'string' },
              label: { type: 'string' },
              filename: { type: 'string' },
              url: { type: 'string' },
              mimeType: { type: 'string' },
            },
          },
        },
      },
    }
  }

  static get jsonAttributes() {
    return ['meta,articleIds', 'meta,publicationDates', 'meta,fundingGroup']
  }

  static get relationMappings() {
    const File = require('../file/data-access')
    const Organization = require('../organization/data-access')
    const Note = require('../note/data-access')
    const Journal = require('../journal/data-access')
    const Review = require('../review/data-access')
    const Team = require('../team/data-access')
    const User = require('../user/data-access')
    const Audit = require('../audit/data-access')
    return {
      reviews: {
        relation: Model.HasManyRelation,
        modelClass: Review,
        join: {
          from: ['manuscript.id', 'manuscript.version'],
          to: ['review.manuscriptId', 'review.manuscriptVersion'],
        },
      },
      files: {
        relation: Model.HasManyRelation,
        modelClass: File,
        join: {
          from: ['manuscript.id', 'manuscript.version'],
          to: ['file.manuscriptId', 'file.manuscriptVersion'],
        },
      },
      notes: {
        relation: Model.HasManyRelation,
        modelClass: Note,
        join: {
          from: 'manuscript.id',
          to: 'note.manuscriptId',
        },
      },
      journal: {
        relation: Model.BelongsToOneRelation,
        modelClass: Journal,
        join: {
          from: 'manuscript.journalId',
          to: 'journal.id',
        },
      },
      organization: {
        relation: Model.BelongsToOneRelation,
        modelClass: Organization,
        join: {
          from: 'manuscript.organizationId',
          to: 'organization.id',
        },
      },
      teams: {
        relation: Model.HasManyRelation,
        modelClass: Team,
        join: {
          from: 'manuscript.id',
          to: 'team.manuscriptId',
        },
      },
      claiming: {
        relation: Model.HasOneRelation,
        modelClass: User,
        join: {
          from: 'manuscript.claimedBy',
          to: 'users.id',
        },
      },
      audits: {
        relation: Model.HasManyRelation,
        modelClass: Audit,
        join: {
          from: 'manuscript.id',
          to: 'audit.audit_log.manuscriptId',
        },
      },
      users: {
        relation: Model.ManyToManyRelation,
        modelClass: User,
        join: {
          from: 'manuscript.id',
          // ManyToMany relation needs the `through` object
          // to describe the join table.
          through: {
            modelClass: Team,
            from: 'team.manuscriptId',
            to: 'team.userId',
          },
          to: 'users.id',
        },
      },
    }
  }

  async saveWithUserAndRole(options, userId, roleName) {
    const Team = require('../team/data-access')
    let trx
    let saved
    try {
      trx = await transaction.start(knex)
      saved = await this.saveWithTrx(trx, options)

      await new Team({
        manuscriptId: saved.id,
        userId,
        roleName,
        updatedBy: userId,
      }).saveWithTrx(trx)

      await trx.commit()
    } catch (error) {
      await trx.rollback()
      logger.error('Nothing was inserted')
      logger.error(error)
      throw error
    }

    return saved
  }

  static async selectById(id, eager = false, trx) {
    let manuscripts
    if (eager) {
      // don't whereNull deleted here
      manuscripts = await Manuscript.query(trx)
        .select('manuscript.*')
        .where('manuscript.id', id)
        .eager('[journal, organization, teams.users, notes, files, claiming]')
        .modifyEager('teams', builder => {
          builder.whereNull('deleted')
        })
        .modifyEager('notes', builder => {
          builder.whereNull('deleted')
        })
        .modifyEager('files', builder => {
          builder.whereNull('deleted')
        })
    } else {
      manuscripts = await Manuscript.query().where('id', id)
    }
    return manuscripts && manuscripts.length > 0 ? manuscripts[0] : null
  }

  static async selectByStatus(
    statuses,
    page = 0,
    pageSize = PAGE_SIZE,
    user,
    claimedBy,
    orgId,
  ) {
    const { isAdmin, manuscriptIds } = await Manuscript.isAdmin(user)
    const manuscripts = await Manuscript.query()
      .with('base_query', q => {
        q.select('*')
          .from('manuscript')
          .modify(q => {
            if (!isAdmin) {
              q.whereIn('manuscript.id', manuscriptIds)
            } else if (orgId) {
              q.where('manuscript.organization_id', '=', orgId)
            }
          })
          .whereIn('manuscript.status', statuses)
          .whereNull('manuscript.deleted')
      })
      .select('*')
      .from(sq => {
        sq.from('base_query')
          .modify(q => {
            if (isAdmin && claimedBy) {
              q.orderByRaw(
                `CASE WHEN "claimed_by" = '${user}' THEN 1 WHEN "claimed_by" is null THEN 2 ELSE 3 END`,
              )
            }
          })
          .orderBy('updated', 'desc')
          .as('bq')
      })
      .modify(q => {
        if (claimedBy) {
          q.leftOuterJoin(
            sq => {
              sq.select(raw('count("id"), "claimed_by"'))
                .from('base_query')
                .groupBy('claimed_by')
                .as('counts')
            },
            join => {
              join.on(
                raw(
                  'bq.claimed_by = counts.claimed_by or (counts.claimed_by is null and bq.claimed_by is null)',
                ),
              )
            },
          )
        }
      })
      .eager('[teams.users, claiming, journal, organization, notes]')
      .modifyEager('teams', e => {
        e.whereNull('deleted')
      })
      .modifyEager('notes', e => {
        e.whereNull('deleted')
      })
      .modify(q => {
        if (page > -1) {
          q.page(page, pageSize)
        }
      })
    logger.debug('manuscripts: ', manuscripts)
    return manuscripts
  }

  static async countByStatus(statuses, user) {
    const { isAdmin, manuscriptIds } = await Manuscript.isAdmin(user)

    const count = await knex('manuscript')
      .count('*')
      .where('status', '=', statuses)
      .modify(q => {
        if (!isAdmin) {
          q.whereIn('id', manuscriptIds)
        }
      })
      .whereNull('deleted')

    return parseInt(count[0].count, 10)
  }

  static async countPreprintsByStatus(statuses, orgId) {
    const count = await knex('manuscript')
      .count('*')
      .where('status', '=', statuses)
      .where('organizationId', '=', orgId)
      .whereNull('deleted')
    return parseInt(count[0].count, 10)
  }

  static async checkAge(statuses, interval, orgId) {
    const rawString = `"updated" < NOW() - INTERVAL '${interval}'`
    const alert =
      (await knex('manuscript')
        .count('*')
        .where('organizationId', '=', orgId)
        .whereIn('status', statuses)
        .whereRaw(rawString)
        .whereNull('deleted')) || '0'
    return parseInt(alert[0].count, 10) > 0
  }

  static async countDeleted(user) {
    const { isAdmin } = await Manuscript.isAdmin(user)

    const count = isAdmin
      ? await knex('manuscript')
          .count('*')
          .whereNotNull('deleted')
      : [{ count: 0 }]

    return parseInt(count[0].count, 10)
  }

  static async getDeleted(page = 0, pageSize = PAGE_SIZE, user) {
    const manuscripts = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .eager('[teams.users, claiming, journal, organization]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .whereNotNull('manuscript.deleted')
      .orderBy('manuscript.updated', 'desc')
      .modify(q => {
        if (page > -1) {
          q.page(page, pageSize)
        }
      })
    logger.debug('manuscripts: ', manuscripts)
    return manuscripts
  }

  static async selectActivityById(id) {
    const manuscripts = await Manuscript.query()
      .where('manuscript.id', id)
      .eager(
        '[journal, organization, notes, claiming, teams.users, audits.user.identities]',
      )
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('audits', q => {
        q.orderBy('audit_log.created', 'asc')
      })
    return manuscripts && manuscripts.length > 0 ? manuscripts[0] : null
  }

  static async selectAllEagerById(id, trx) {
    const manuscripts = await Manuscript.query(trx)
      .select('manuscript.*')
      .where('manuscript.id', id)
      .whereNull('deleted')
      .eager(
        '[journal, organization, teams.users, notes, files, audits, claiming]',
      )
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('files', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('audits', q => {
        q.orderBy('audit_log.created', 'asc')
      })
    return manuscripts && manuscripts.length > 0 ? manuscripts[0] : null
  }

  static async getVersions(idList, trx) {
    return Manuscript.query(trx)
      .select('manuscript.id', 'manuscript.version')
      .whereIn('manuscript.id', idList)
      .whereNull('deleted')
      .orderBy('manuscript.version')
  }

  static async selectByPdfDepositStatesNull() {
    const manuscripts = Manuscript.query()
      .whereNull('pdf_deposit_state')
      .whereNull('deleted')
    logger.debug('manuscripts: ', manuscripts)
    return manuscripts
  }

  static async selectByPdfDepositStates(depositStates) {
    const manuscripts = Manuscript.query()
      .whereIn('pdf_deposit_state', depositStates)
      .whereNull('deleted')
      .eager('[files]')
      .modifyEager('files', builder => {
        builder.whereNull('deleted')
      })
    logger.debug('manuscripts: ', manuscripts)
    return manuscripts
  }

  static async selectAll(user) {
    const manuscripts = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .leftJoin('team', 'team.manuscript_id', 'manuscript.id')
      .groupBy('manuscript.id', 'team.user_id', 'team.role_name')
      .whereNull('team.deleted')
      .where('team.user_id', user)
      .whereNull('manuscript.deleted')
      .orderBy('manuscript.updated', 'desc')
      .eager('[teams.users, journal, organization, notes]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
    return manuscripts
  }

  static async searchArticleIds(id, user, trx) {
    const { isAdmin, manuscriptIds } = await Manuscript.isAdmin(user)

    const manuscripts = await Manuscript.query(trx)
      .where(
        raw('lower("meta,article_ids"::text)::jsonb @> lower(?)::jsonb', [
          `[{"id": "${id}"}]`,
        ]),
      )
      .modify(q => {
        if (!isAdmin) {
          q.whereIn('id', manuscriptIds)
        }
      })
      .whereNull('manuscript.deleted')
      .eager('[journal, notes, organization]')
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
    return manuscripts
  }

  static async searchByTitle(query, user) {
    const { isAdmin, manuscriptIds } = await Manuscript.isAdmin(user)

    const manuscripts = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .where(
        raw(`(? ilike concat("meta,title", '%') or "meta,title" ilike ?)`, [
          query,
          `${query}%`,
        ]),
      )
      .whereNull('manuscript.deleted')
      .modify(q => {
        if (!isAdmin) {
          q.whereIn('id', manuscriptIds)
        }
      })
      .eager('[teams.users, claiming, journal, organization, notes]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
      .orderBy('manuscript.updated', 'desc')
    return manuscripts
  }

  static async searchByTitleOrUser(
    query,
    page = 0,
    pageSize = PAGE_SIZE,
    user,
  ) {
    const { isAdmin, manuscriptIds } = await Manuscript.isAdmin(user)

    const manuscripts = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .modify(q => {
        if (!isAdmin) {
          q.whereIn('manuscript.id', manuscriptIds)
        }
      })
      .whereNull('manuscript.deleted')
      .eager('[teams.users, claiming, journal, organization, notes]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
      .join('team', 'team.manuscript_id', 'manuscript.id')
      .join('users', 'team.user_id', 'users.id')
      .join('identity', 'identity.user_id', 'users.id')
      .where(
        raw(
          `("identity"."email" ilike ? or "users"."surname" ilike ? or "meta,title" ilike ?)`,
          [query, `${query}%`, `${query}%`],
        ),
      )
      .orderBy('manuscript.updated', 'desc')
      .page(page, pageSize)

    return manuscripts
  }

  static async getNcbiReady() {
    const manuscripts = await Manuscript.query()
      .where('status', 'repo-processing')
      .where('ebiState', 'EBI received')
      .whereNull('ncbiState')
      .whereNull('deleted')
      .eager('[journal, files, organization, teams.users.identities]')
    logger.debug('manuscripts: ', manuscripts)
    return manuscripts
  }

  static async getPublishCheckReady() {
    const manuscripts = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .eager('[users.[roles, identities], audits]')
      .where('manuscript.status', 'repo-processing')
      .where('manuscript.ncbiState', 'success')
      .whereNull('manuscript.deleted')
      .modifyEager('audits', q => {
        q.whereRaw("changes->>'status' = 'published'")
      })

    const preprints = await Manuscript.query()
      .distinct('manuscript.id')
      .select('manuscript.*')
      .eager('[users.[roles, identities], audits]')
      .where('manuscript.status', 'repo-processing')
      .where('manuscript.ebiState', 'success')
      .whereNull('manuscript.deleted')
      .modifyEager('audits', q => {
        q.whereRaw("changes->>'status' = 'published'")
      })

    return { manuscripts, preprints }
  }

  static async getDeletionReady() {
    return Manuscript.query()
      .where('status', 'being-withdrawn')
      .where('ncbiState', 'ncbi-withdraw')
      .whereNull('manuscript.deleted')
      .eager('[notes]')
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
  }

  static async getIncomplete() {
    return Manuscript.query()
      .whereIn('status', ['INITIAL', 'READY'])
      .whereRaw(
        '("updated"::date = current_date - interval \'1 day\' or "updated"::date = current_date - interval \'7 days\')',
      )
      .whereNull('manuscript.deleted')
      .eager('[teams.users.identities]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
  }

  static async getNeedFix() {
    return Manuscript.query()
      .where('status', 'submission-error')
      .whereRaw(
        '("updated"::date = current_date - interval \'1 week\' or "updated"::date = current_date - interval \'2 weeks\')',
      )
      .whereNull('manuscript.deleted')
      .eager('[notes, teams.users.identities]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
  }

  static async getStalled() {
    return Manuscript.query()
      .whereIn('status', ['in-review', 'xml-review'])
      .whereRaw(
        '("updated"::date = current_date - interval \'1 week\' or "updated"::date = current_date - interval \'2 weeks\')',
      )
      .whereNull('manuscript.deleted')
      .eager('[notes, teams.users.identities, organization]')
      .modifyEager('teams', builder => {
        builder.whereNull('deleted')
      })
      .modifyEager('notes', builder => {
        builder.whereNull('deleted')
      })
  }

  static async insert(manuscript) {
    const { id } = await Manuscript.query().insertGraph(manuscript)
    const manuscriptInserted = await Manuscript.query()
      .where('id', id)
      .whereNull('deleted')
      .eager()
    return manuscriptInserted[0]
  }

  static async delete(id, userId) {
    return Manuscript.query()
      .patchAndFetchById(id, {
        deleted: new Date().toISOString(),
        updatedBy: userId,
      })
      .returning('*')
  }

  static async isAdmin(user) {
    const Team = require('../team/')
    const roles = await Team.selectByUserId(user)
    const isAdmin = roles.some(r =>
      ['admin', 'external-admin', 'tagger'].includes(r.role),
    )
    const manuscriptIds = roles
      .map(r => r.objectId)
      .filter(objectId => objectId)
    return { isAdmin, manuscriptIds }
  }
}

module.exports = Manuscript
